#!/usr/bin/env python
# -*- coding: utf-8 -*-
# AUTHOR: XYCoder
# FILE: /Users/v-sunweiwei/Desktop//xyscript/xyscript/ios/gitandjira.py
# DATE: 2019/08/16 Fri
# TIME: 17:33:35

# DESCRIPTION:gitlab相关操作与jira相关操作

import os,sys,re,json,datetime
from xyscript.common.xylog import *
from xyscript.common.mail import Email

LOGNUM = 10
LASTSUCCESS = None
CURRENTSUCCESS = None

class GitLab:
    def get_all_commit_between_push(self,code_branch,package_branch,workspace,mail,url=None,keyword=None):
        commit_result = {}
        os.chdir(workspace)
        commit_content = "merge " + code_branch + " into " + package_branch
        log_num = self.get_num_of_need_commit(commit_content,workspace)

        now = datetime.datetime.now()
        otherStyleTime = now.strftime("%Y-%m-%d %H:%M:%S")

        result = os.popen('git log -' + str(log_num) + ' --before="'+ otherStyleTime +'" --pretty=format:"%cn<%ce>%ce%cd%s" --date=iso')
        # print('git log -' + str(log_num) + ' --before="'+ otherStyleTime +'" --pretty=format:"%cn<%ce>%ce%cd%s" --date=iso')
        all_lines = result.readlines()
        all_commits = []
        for line in all_lines:
            info = line.split('')
            commit_info = {}
            commit_info['user'] = info[0]
            commit_info['date'] = info[2]
            commit_info['info'] = info[3].split('\n')[0]
            commit_info['email'] = info[1]
            all_commits.append(commit_info)
        
        last_commit_date = otherStyleTime
        # print(all_lines)
        old_commit_date = all_commits[-1]['date']
        cc = all_commits[0]['email']
        submodule_commit_list = []
        submodule_list = self._get_current_config_file()
        for moduleConfig in submodule_list:
            module_path = workspace + "/" + moduleConfig["module"]
            commit_array = self.get_submodule_changes_during(module_path,old_commit_date,last_commit_date)
            if len(commit_array) > 0:
                submodule_commit_list.append({'name':moduleConfig["module"],'commit_log':commit_array})

        
        public_module_list = []
        pod_path = workspace + '/' + 'Pods/'
        if os.path.exists(pod_path) and keyword is not None and len(keyword) >0:
            for root, dirs, files in os.walk(pod_path):
                for d in dirs:
                    if self._is_private_module(d,keyword):
                        commit_array = self.get_submodule_changes_during(module_path,old_commit_date,last_commit_date)
                        if len(commit_array) > 0:
                            public_module_list.append({'name':d,'commit_log':commit_array})
        
        commit_result['cc'] = cc
        commit_result['project_name'] = workspace.split('/')[-1]
        commit_result['start_time'] = last_commit_date
        commit_result['end_time'] = old_commit_date
        commit_result['code_branch'] = code_branch
        commit_result['package_branch'] = package_branch
        commit_result['shell'] = all_commits
        commit_result['submodule'] = submodule_commit_list
        commit_result['private_module'] = public_module_list
        if url is not None:
            commit_result['jiraurl'] = url
            
        Email(mail).send_merge_result(commit_result)

    def _is_private_module(self,dir,keywords):
        for keyword in keywords:
            if dir.startswith(keyword):
                return True
        return False

    
    def get_submodule_changes_during(self,workspace,start_time,end_time):
        os.chdir(workspace)
        result = os.popen('git log --date=iso --before="'+ end_time +'" --after="' + start_time + '" --pretty=format:"%cn%cd%s"')
        all_lines = result.readlines()
        all_commits = []
        for line in all_lines:
            info = line.split('')
            commit_info = {}
            commit_info['user'] = info[0]
            commit_info['date'] = info[1]
            commit_info['info'] = info[2].split('\n')[0]
            all_commits.append(commit_info)
        return all_commits

    def _get_current_config_file(self):
        file = open("ProjConfig.json")
        moduleConfigList = json.load(file)
        return moduleConfigList
    
    def get_jira_url_with_commit(self,commit_info):
        pass

    def get_current_branch_name(self,workspace):
        os.chdir(workspace)
        result = os.popen('git branch')
        all_lines = result.readlines()
        for line in all_lines:
            if '* ' in line:
                return line[2:]
        return None

    def get_num_of_need_commit(self,content,workspace):
        global LOGNUM
        global LASTSUCCESS
        global CURRENTSUCCESS
        os.chdir(workspace)
        result = os.popen('git log -' + str(LOGNUM) + ' --pretty=format:"%cn<%ce>%cd%s"')
        all_lines = result.read()

        content_num = all_lines.count(content)

        # merge = re.compile('Merge\: ([a-z0-9]{7,})+([\t ]*)+([a-z0-9]{7,})')
        # merge_result = merge.findall(all_lines)
        # merge_num = len(merge_result)
        if content_num >=2:
            CURRENTSUCCESS = True
        else:
            CURRENTSUCCESS = False

        if LASTSUCCESS == True and CURRENTSUCCESS:
            LOGNUM = LOGNUM - 1
        elif LASTSUCCESS == False and CURRENTSUCCESS:
            return LOGNUM
        elif LASTSUCCESS == True and not CURRENTSUCCESS:
            return LOGNUM + 1
        elif LASTSUCCESS == False and not CURRENTSUCCESS:
            LOGNUM = LOGNUM + 1
        elif LASTSUCCESS is None and CURRENTSUCCESS:
            LOGNUM = LOGNUM - 1
        elif LASTSUCCESS is None and not CURRENTSUCCESS:
            LOGNUM = LOGNUM + 1

        LASTSUCCESS = CURRENTSUCCESS
        return self.get_num_of_need_commit(content,workspace)



if __name__ == "__main__":
    # 测试代码
    GitLab().get_all_commit_between_push(code_branch='Develop',package_branch='zuche-test',workspace='/Users/v-sunweiwei/Desktop//ios-shell-passenger',mail='m18221031340@163.com')
    # now = datetime.datetime.now()
    # print(now)
    # otherStyleTime = now.strftime("%Y-%m-%d %H:%M:%S")
    # print(otherStyleTime) 


# -*- coding: utf-8 -*-
import sys
from xyscript.config import Config
from setup import setup as st
def main():
    print("进入入口主程序")
    pass

def test():
    pass

if __name__ == '__main__':
    main()
    print(st)
    # print(Config().get_version())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os,random
from xyscript.common.xylog import *

class IOSTool:
    def get_file_with_name(self,file_name,folder):
        for root, dirs, files in os.walk(folder):
            if file_name in files:
                root = str(root)
                dirs = str(dirs)
                print(root)
                
                return os.path.join(root, file_name)
        return None
        
    def get_file_end_count(self, file_path, old_str):
        Ropen=open(file_path,'r')#读取文件
        flagCount = 0
        for line in Ropen:
            if old_str in line:#如果.h文件中的某一行里包含old_str,则往这一行添加一下语句
                flagCount += 1
        return flagCount  
    
    # 产生一个satrtIndex到endIndex位长度的随机字符串
    def get_randomStr(self,satrtIndex,endIndex):
        numbers = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
        # random.choice()从列表中返回一个随机数
        final = (random.choice(numbers))
        # 从(50,100)列表中取出一个随机数
        index = random.randint(satrtIndex, endIndex)
        for i in range(index):
            final += (random.choice(numbers))
        return final

    def add_code(self, file_path, old_str, endTotalCount):
        # .h文件里属性的类型从这个数组里随机选
        file_data = ""
        Ropen=open(file_path,'r')
        flagCount = 0
        for line in Ropen:
            className = 'NSString'
            nameStr = IOSTool().get_randomStr(6,10)

            if old_str in line:
                flagCount += 1
                if flagCount==endTotalCount:
                    file_data += '\n@property(nonatomic,strong) '+className +' *'+nameStr+';\n'
                file_data += line
            else:
                file_data += line
        Ropen.close()
        Wopen=open(file_path,'w')
        Wopen.write(file_data)
        Wopen.close()

    def add_content_to_file(self,file_name):
        try:
            file = IOSTool().get_file_with_name(file_name,os.getcwd())
            if file == None:
                # faillog("cannot find a file named " + file_name)
                #🚀找不到就算了，什么都不展示,别露出破绽🚀
                #但是功能还是需要的，
                pass
            else:
                end_count = IOSTool().get_file_end_count(file,"@end")
                IOSTool().add_code(file,"@end",end_count)
                successlog("add code to " + file + " success")
        except BaseException as error:
            faillog("add code to " + file_name + "failed:" + str(error))
        
    # TODO (m18221031340@163.com) 清理项目中没有用到的图片
    def clear_unused_picture(self):
        pass
    
    # TODO (m18221031340@163.com) 项目瘦身
    def thin_project(self):
        pass

    # TODO (m18221031340@163.com) 自动测试 Appium
    def auto_test(self):
        pass

    

if __name__ == "__main__":
    IOSTool().add_content_to_file("AppDelegate.h")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import base64
import HTMLParser
import cgi
 
def html2py(html_name):
    """
    将图像文件转换为py文件
    :param picture_name:
    :return:
    """
    html_parser = HTMLParser.HTMLParser()
    html = open("%s" % html_name, 'rb')
    html_text = html.read()
    txt = html_parser.unescape(html_text)
    html.close()
    write_data = '%s = "%s"' % ((html_name.split('.')[0]).split('/')[-1],txt)
    f = open('/Users/v-sunweiwei/Desktop//xyscript/xyscript/resource/html/html.py', 'r+')
    date = f.read()
    f = open('/Users/v-sunweiwei/Desktop//xyscript/xyscript/resource/html/html.py', 'w+')
    f.write(date + '\n\r' + write_data)
    f.close()

 
if __name__ == '__main__':
    htmls = ["/Users/v-sunweiwei/Desktop//xyscript/xyscript/resource/html/diagnosemail.html"]
    for i in htmls:
        html2py(i)



from functools import wraps

import cProfile

import time


def func_time(f):
    """
    简单记录执行时间
    :param f:
    :return:
    """

    @wraps(f)
    def wrapper(*args, **kwargs):
        start = time.time()
        result = f(*args, **kwargs)
        end = time.time()
        print(f.__name__, 'took', end - start, 'seconds')
        return result

    return wrapper


def func_cprofile(f):
    """
    内建分析器
    """

    @wraps(f)
    def wrapper(*args, **kwargs):
        profile = cProfile.Profile()
        try:
            profile.enable()
            result = f(*args, **kwargs)
            profile.disable()
            return result
        finally:
            profile.print_stats(sort='time')

    return wrapper



try:
    from line_profiler import LineProfiler

    def func_line_time(follow=[]):
        """
        每行代码执行时间详细报告
        :param follow: 内部调用方法
        :return:
        """
        def decorate(func):
            @wraps(func)
            def profiled_func(*args, **kwargs):
                try:
                    profiler = LineProfiler()
                    profiler.add_function(func)
                    for f in follow:
                        profiler.add_function(f)
                    profiler.enable_by_count()
                    return func(*args, **kwargs)
                finally:
                    profiler.print_stats()

            return profiled_func

        return decorate

except ImportError:
    def func_line_time(follow=[]):
        "Helpful if you accidentally leave in production!"
        def decorate(func):
            @wraps(func)
            def nothing(*args, **kwargs):
                return func(*args, **kwargs)

            return nothing

        return decorate



import os,time
class MyFrame(wx.Frame):
    
    #设置默认delay时间值
    delayDefault = "2"
    #设置默认种子数
    seedDefault = "5000000"
    #设置默认执行次数
    executionFrequencyDefault = "60000000"
    logDir = "./"
    def __init__(self):
        #执行方式定义
            excuteMode = ["忽略程序崩溃",
                        "忽略程序无响应",
                        "忽略安全异常",
                        "出错中断程序",
                        "本地代码导致的崩溃",
                        "默认"
                        ]
            #日志输出等级区分
            logMode = ["简单","普通","详细"]
            executionModeDefault = excuteMode[0]
            #初始化菜单按钮
            menuBar = wx.MenuBar()
            menu1 = wx.Menu("")
            menuBar.Append(menu1, "File")
            self.SetMenuBar(menuBar)
            #初始化标签栏
            wx.StaticText(panel, -1, "种子数:", pos=(xPos, yPos))
            self.seedCtrl = wx.TextCtrl(panel, -1, "", pos=(xPos1, yPos))
            #绑定点击事件
            self.seedCtrl.Bind(wx.EVT_KILL_FOCUS, self.OnAction)
            self.seedCtrl.SetFocus()
            #初始化标签栏
            wx.StaticText(panel, -1, "执行次数:", pos=(xPos, yPos+yDelta))
            #设置窗口位置
            self.excuteNumCtrl = wx.TextCtrl(panel, -1, "", pos=(xPos1, yPos+yDelta))
            #初始化标签栏
            wx.StaticText(panel, -1, "延时:", pos=(xPos, yPos+2*yDelta))
            self.delayNumCtrl = wx.TextCtrl(panel, -1, "", pos=(xPos1, yPos+2*yDelta))
            #初始化标签栏       
            wx.StaticText(panel, -1, "执行方式:", pos=(xPos, yPos+3*yDelta))
            #设置窗口位置
            self.excuteModeCtrl = wx.ComboBox(panel, -1, "", (xPos1,yPos+3*yDelta), choices=excuteMode,style=wx.CB_DROPDOWN)
    #设置初始化checklistbox，下拉菜单
            self.checkListBox = wx.CheckListBox(panel, -1, (xPos, yPos+4*yDelta ), (400, 350), [])
            wx.StaticText(panel, -1, "日志输出等级:", pos=(xPos, yPoslayout-yDelta))
            self.logModeCtrl = wx.ComboBox(panel, -1, "", (xPos1,yPoslayout-yDelta), choices=logMode,style=wx.CB_DROPDOWN)
    
            #初始化按钮，读取程序包按钮绑定readButton事件
            self.readButton = wx.Button(panel, -1, "读取程序包", pos=(xPos, yPoslayout))
            self.Bind(wx.EVT_BUTTON, self.OnReadClick, self.readButton)
            self.readButton.SetDefault()
            #初始化默认参数按钮，绑定defaultButton事件
            self.defaultButton = wx.Button(panel, -1, "默认参数", pos=(xPos, yPoslayout+yDelta))
            self.Bind(wx.EVT_BUTTON, self.OnResetClick, self.defaultButton)
            self.defaultButton.SetDefault()
            #初始化一键monkey按钮，按钮绑定quick事件
            self.quickButton = wx.Button(panel, -1, "一键Monkey", pos=(xPos+120, yPoslayout+yDelta))
            self.Bind(wx.EVT_BUTTON, self.OnQuickStartClick, self.quickButton)
            self.quickButton.SetDefault()
#生成log函数

    def OnBuildLog(self,event):
        os.chdir(self.logDir)  
        #通过日期创建唯一标识文件名称
        date = time.strftime('%Y-%m-%d-%H-%M',time.localtime(time.time()))
        dir_m = "Monkey_Log_"+date.replace("-","")
        dir0 = "sdcard0_log"
        创建目标文件目录
        if (os.path.exists(dir_m+"/"+dir0)):
            print ("already exists")
        else:
            os.system("mkdir -p "+dir_m+"/"+dir0)
        os.chdir(dir_m)
        #通过adb命令导出log文件到目标文件夹中
        os.system("adb pull /storage/sdcard0/log/ "+dir0)
        #查找异常log文件
        self.BuildFatalLog(os.getcwd())

    #遍历所有的log文件函数
def ListFiles(self,path):
    #遍历文件件
        for root,dirs,files in os.walk(path):
            log_f = ""
            for f in files:
                if(f.find("main") == 0):
                    log_f = f.strip()
                    #切换到目标目录
                    os.chdir(root)
                    #通过grep 命令查找所有的异常文件
                    if (log_f != ""):
                        grep_cmd = "grep-Eni-B2-A20'FATAL|error|exception|system.err|androidruntime' " + log_f + " > " + log_f + "_fatal.log"
                        os.system(grep_cmd)

    #查找异常文件函数
def BuildFatalLog(self,path):
    self.ListFiles(path)

#读取程序包函数声明
def OnReadClick(self, event):
    #清空控件内容
        self.checkListBox.Clear()
        #通过读取手机data/data目录来确认所有的包名
        os.system("adb shell ls data/data > ~/log.log")
        #解析log.log文件
        home = os.path.expanduser('~')
        f = open(home+"/log.log", 'r')
        line = f.readline()
        while line:
            line = f.readline()
            if (line != ""):
                print ("===="+line)
                #将解析的包名，添加包名checkbox中显示
                self.checkListBox.Append(line)
        f.close()


if __name__ == '__main__':
    pass
    




#-*- encoding:utf-8 -*-
import os, sys

class Common:
    def run_shellstring(self,shell_str):
        os.system(shell_str)
    
    def run_shellstring_forreslut(self,shell_str):
        os.system(shell_str)
        result = os.popen(shell_str)
        text = result.read()
        return text
        


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import base64
 
def pic2py(picture_name):
    """
    将图像文件转换为py文件
    :param picture_name:
    :return:
    """
    open_pic = open("%s" % picture_name, 'rb')
    b64str = base64.b64encode(open_pic.read())
    open_pic.close()
    # 注意这边b64str一定要加上.decode()
    write_data = '%s = "%s"' % ((picture_name.split('.')[0]).split('/')[-1],b64str.decode())
    f = open('/Users/v-sunweiwei/Desktop//xyscript/xyscript/resource/img/image.py', 'r+')
    date = f.read()
    f = open('/Users/v-sunweiwei/Desktop//xyscript/xyscript/resource/img/image.py', 'w+')
    f.write(date + '\n\r' + write_data)
    f.close()
 
if __name__ == '__main__':
    pics = ["/Users/v-sunweiwei/Desktop//xyscript/xyscript/resource/img/jsonfileicon.png"]
    for i in pics:
        pic2py(i)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# AUTHOR: XYCoder
# FILE: ~/Desktop//xyscript/xyscript/test/test.py
# DATE: 2019/08/21 Wed
# TIME: 14:12:21

# DESCRIPTION:


# -*- coding: GB2312 -*-
import os,sys,json,wx
import os.path
import time
from threading import Timer
from threading import Thread
import glob
from xyscript.common.sysenv import System
from xyscript.common.xylog import *
from concurrent.futures import ThreadPoolExecutor, as_completed
from xyscript.common.interaction import Interaction
from xyscript.common.file import File
# from xyscript.test.testUI.te import TestThread

# 获取连接设备
DEVICES = ['xytest']
# 种子数
SEED_NUM = 23
# 延时
TIME_DELAY = 300
# 执行次数
RUN_TIMES = 500000
# 执行方式 --ignore-crashes --ignore-timeouts --ignore-security-exceptions --ignore-native-crashes 
IMPLEMENT_WAY = ['ignore-crashes','ignore-timeouts','ignore-security-exceptions','ignore-native-crashes']
# 日志级别
LOG_LEVEL = ' -v -v -v'
# 日志地址
LOG_ADDRESS = None
# 执行周期
LIFECYCLE = 4
# 执行次数
CYCLES = 3
# 所有在测试的包名
PACKAGES_TESTING = []
# 生成配置文件名称
CONFIG_NAME = 'config.json'
# 当前执行的次数
CURRENT_INDEX = 1
# 线程数
THREAD_NUM = 0



platform = System().get_platform()

class Test:
    def start_test(self):
        ui = TestUI()
        try:
            ui.get_seed_num()
            ui.get_run_times()
            ui.get_run_cycle()
            ui.get_link_mobile()
            self.setup_task_files(ui)
            self.run_monkey_with_config(ui)
        except BaseException as error:
            faillog(format(error))
            # 停掉所有测试进程
            mobile = ui.get_config_with_name('mobile')
            for key,value in mobile.items():
                dev = key.split('_')[-1]
                self.kill_monkey_with_package_name(dev)
                self.kill_logcat(dev)
            sys.exit()

    def stop_test(self,ui,t=None,t0=None):
        mobile = ui.get_config_with_name('mobile')
        for key,value in mobile.items():
            dev = key.split('_')[-1]
            self.kill_monkey_with_package_name(dev)
            self.kill_logcat(dev)
            if t is not None:
                t.join()
            if t0 is not None:
                t0.stop_thread()
            

    def setup_task_files(self,ui):
        data = ui.get_config_with_name()
        mobile_dict = data.get('mobile')
        workspace = data.get('workspace')
        for key,value in mobile_dict.items():
            mobile_folder_path = workspace + key + '/'
            if platform in ["Windows/Cygwin","Windows"]:
                mobile_folder_path = mobile_folder_path.replace('/','\\')
            mobile_folder_path = mobile_folder_path.replace(' ','-')
            if not os.path.exists(mobile_folder_path):
                File().creat_folder(mobile_folder_path)
            # for package_name in value:
            #     package_path = mobile_folder_path + package_name + '/'
            #     if not os.path.exists(package_path):

            #         File().creat_folder(package_path)
        
        logcat_path = workspace + 'logcat/'
        if platform in ["Windows/Cygwin","Windows"]:
            logcat_path = logcat_path.replace('/','\\')
        if not os.path.exists(logcat_path):
            File().creat_folder(logcat_path)
            ui.set_config_with_name('logcat_path',logcat_path)

        
    def run_monkey_with_config(self,ui):
        global CURRENT_INDEX
        timer_interval = 3
        # start_time = time.time()
        executor = ThreadPoolExecutor(max_workers=100)

        mobile_dict = ui.get_config_with_name('mobile')
        array = []
        for key,value in mobile_dict.items():
            array.append(key)
        # 循环次数
        cycles = ui.get_config_with_name('cycles')

        all_task = [executor.submit(self.run_adb_str,(ui),(key)) for (index,key) in enumerate(array)]

        # t = Timer(timer_interval,change_process,[time.time() - start_time,CURRENT_INDEX])
        # t.start()
        over_num = 0
        for index,future in enumerate(as_completed(all_task)):
            # sys.stdout.write("\r进度:%d / %d" %(index+1, len(all_task)))
            # sys.stdout.flush()
            data = future.result()
            if data is None:
                over_num = over_num + 1
                if over_num == len(array):
                    # t.cancel()
                    CURRENT_INDEX = CURRENT_INDEX + 1
                    if CURRENT_INDEX <= cycles:
                        self.run_monkey_with_config(ui)  
        

    def run_adb_str(self,ui,key):
        global CURRENT_INDEX
        # 序列号
        dev = key.split('_')[-1]
        # logcat
        logcat_file_name = key + str(CURRENT_INDEX) + '.log'
        logcat_file_path = ui.get_config_with_name('logcat_path')
        os.popen('adb -s ' + dev + ' shell logcat -c') #清楚日志
        os.popen('adb -s ' + dev + ' shell  logcat -v time > ' + logcat_file_path + logcat_file_name) #记录日志
        # 包名
        packages = ui.get_config_with_name('mobile').get(key)
        p_array = ''
        for p in packages:
            p_array = p_array + ' -p ' + p
        # 种子数
        seed_num = ui.get_config_with_name('seed_num')
        # 模式
        implement_way = ui.get_config_with_name('implement_way')
        run_mode = ''
        for behav in implement_way:
            run_mode = run_mode + ' --' + behav
        # 延迟
        time_delay = ui.get_config_with_name('time_delay')
        # 日志等级
        log_level = ui.get_config_with_name('log_level')
        # 执行次数
        run_times = ui.get_config_with_name('run_times')
        # 日志地址
        workspace = ui.get_config_with_name('workspace')
        log_path = workspace + key + '/'  + key + '_' +str(CURRENT_INDEX) + '.log'
        if  platform in ["Windows/Cygwin","Windows"]:
            log_path = log_path.replace('/','\\')
        log_path = log_path.replace(' ','-')
        
        shell_str = 'adb -s '+ dev +' shell monkey ' + p_array + ' -s ' + str(seed_num) + run_mode + ' --throttle ' + str(time_delay) + ' --pct-touch 40 --pct-motion 25 --pct-appswitch 15 --pct-trackball 5 --pct-majornav 5 ' + log_level + ' ' + str(run_times) + ' >' + log_path + ' 2>&1'
        result =os.popen(shell_str)

        # 每次执行周期
        cycle = ui.get_config_with_name('life_cycle')
        run_time_delay = cycle * 60 * 60
        time.sleep(int(run_time_delay))
        self.kill_monkey_with_package_name(dev)
        self.kill_logcat(dev)

    def kill_logcat(self,dev):
        result = None
        if  platform in ["Windows/Cygwin","Windows"]:
            result= os.popen('adb -s ' + dev + ' shell ps | findstr "logcat"').readlines()
        elif platform in ["Mac OS X","OS/2","OS/2 EMX","Linux"]:
            result= os.popen('adb -s ' + dev + ' shell ps | grep logcat').readlines()

        for line in result:
            pid_str_array = line.split(' ')
            if len(pid_str_array) >= 6:
                pid = pid_str_array[5]
                os.popen('adb -s ' + dev + ' shell kill -9 ' + str(pid)).readlines()

        
    def kill_monkey_with_package_name(self,dev,packages=None):
        adb_result = None
        if  platform in ["Windows/Cygwin","Windows"]:
            adb_result= os.popen('adb -s ' + dev + ' shell ps | findstr "monkey"').readlines()
        elif platform in ["Mac OS X","OS/2","OS/2 EMX","Linux"]:
            adb_result= os.popen('adb -s ' + dev + ' shell ps | grep monkey').readlines()

        for line in adb_result:
            pid_str_array = line.split(' ')
            if len(pid_str_array) >= 6:
                pid = pid_str_array[5]
                os.popen('adb -s ' + dev + ' shell kill ' + str(pid)).readlines()
            

    def get_device_name_with_IMEI(self,IMEI,type):
        result = ''
        # if  platform in ["Windows/Cygwin","Windows"]:
            # adb -s 37KNW18802008494 shell getprop ro.product.brand
        # print('adb -s ' + IMEI + ' shell getprop ro.product.' + type)
        result = os.popen(
            'adb -s ' + IMEI + ' shell getprop ro.product.' + type).readlines() 
        # elif platform in ["Mac OS X","OS/2","OS/2 EMX","Linux"]:
        #     result = os.popen(
        #     "adb -s " + IMEI + ' shell cat /system/build.prop | grep ro.product.' + type +'= ').readlines() 

        if result is not None and len(result)>=1 and len(result[0])>2:
            # print(result[0][:-1])
            return result[0][:-1]
        return None

    def get_package_name_contain(self,IMEI,name=None):
        shell = "adb -s " + IMEI + ' shell pm list packages'
        if name is not None:
            shell = "adb -s " + IMEI + ' shell pm list packages | grep ' + name
            if  platform in ["Windows/Cygwin","Windows"]:
                shell = "adb -s " + IMEI + ' shell pm list packages | findstr ' + name
        
        packages = []
        result = os.popen(shell).readlines()
        for package in result:
            package_str_array = package.split(':')
            if len(package_str_array) >=2:
                packages.append(package.split(':')[1].rstrip())
        return packages

    # def 

class TestUI:
    def __init__(self):
        global SEED_NUM ,TIME_DELAY ,RUN_TIMES ,IMPLEMENT_WAY ,LOG_LEVEL ,LOG_ADDRESS ,LIFECYCLE ,PACKAGES_TESTING ,CONFIG_NAME ,CYCLES
        LOG_ADDRESS = self.get_desktop_path()
        #print(LOG_ADDRESS)
        self.workspace = LOG_ADDRESS
        self.seed_num = SEED_NUM
        self.time_delay = TIME_DELAY
        self.run_times = RUN_TIMES
        self.implement_way = IMPLEMENT_WAY
        self.log_level = LOG_LEVEL
        self.life_cycle = LIFECYCLE
        self.packages_testing = PACKAGES_TESTING
        self.config_name = CONFIG_NAME
        self.cycles = CYCLES
        self.setup_config_file()
# log_store_local = /dist/
# cache_path = /XYCache/
    
    def get_desktop_path(self):
        desktop_path = ''
        cache_path = ''
        sym_path = "monkeytest/"
        if  platform in ["Windows/Cygwin","Windows"]:
            desktop_path = os.path.join(os.path.expanduser("~"), 'Desktop')
            cache_path = desktop_path + '/XYCache/' + sym_path
            cache_path = cache_path.replace('/','\\')
        elif platform in ["Mac OS X","OS/2","OS/2 EMX","Linux"]:
            desktop_path = os.path.expanduser('~')
            cache_path = desktop_path + '/XYCache/' + sym_path

        File().creat_folder(cache_path)
        return cache_path

    def get_workspace(self):
        return Interaction().select_directory(2)


    def get_run_cycle(self):
        default_cycle = self.get_config_with_name('life_cycle')
        cycle = input('请输入每次执行的周期(小时),默认值是 ' + str(default_cycle) + ' 小时:')
        if len(cycle) != 0:
            self.set_config_with_name('life_cycle',float(cycle))

    
    def get_run_times(self):
        default_cycle = self.get_config_with_name('cycles')
        cycles = input('请输入任务循环次数,默认值是 ' + str(default_cycle) + ' 次:')
        if len(cycles) != 0:
            self.set_config_with_name('cycles',int(cycles))

    def get_seed_num(self):
        seed_num = self.get_config_with_name('seed_num')
        seed = input('请输入任务种子数,默认值是 ' + str(seed_num) + ':')
        if len(seed) != 0:
            self.set_config_with_name('seed_num',int(seed))       

    def get_link_mobile(self):
        rt = os.popen('adb devices').readlines()  
        n = len(rt) - 2
        # if UI is not None:
        print ("当前已连接待测手机数为：" + str(n))

        aw = input("是否要继续你的monkey测试，请输入(yes or no): ")

        if aw == 'yes':
            mobile_data = {}
            ds = list(range(n))
            for i in range(n):
                nPos = rt[i + 1].index("\t")
                ds[i] = rt[i + 1][:nPos]
                dev = ds[i]
                # 获取手机型号
                model = Test().get_device_name_with_IMEI(dev,'model')
                # 获取手机名称
                name = Test().get_device_name_with_IMEI(dev,'brand')
                # 包关键字
                package_keyword = input('请输入 ' + model + ' 中需要测试包名关键词:')
                if len(package_keyword) <=0 :
                    package_keyword = None
                # 所有包名
                pk = Test().get_package_name_contain(dev,package_keyword)
                # 需要测试的包名序号
                answer_package_str = '请输入 ' + model + ' 中需要测试包名序号(多个序号按照\',\'分隔):\n'
                for index,package in enumerate(pk):
                    answer_package_str = answer_package_str + str(index + 1) + '.' + package + '\n'
                answer_package_result = input(answer_package_str)
                need_test_package_arr = answer_package_result.split(',')
                mobile_folder_name = name + '_' + model + '_' + dev

                p_array = []
                for index in need_test_package_arr:
                    p_array.append(pk[int(index) -1])

                mobile_data[mobile_folder_name] = p_array

            self.set_config_with_name('mobile',mobile_data)


        elif aw == 'no':
            #print('请重新确认所有待测手机是否已通过adb命令连接至pc!')
            sys.exit()
        else:
            print ("测试结束，输入非法，请重新输入yes or no！")
            sys.exit()

    def get_link_mobile_with_keyword(self,keyword):
        rt = os.popen('adb devices').readlines()  
        n = len(rt) - 2
        mobile_data = {}
        ds = list(range(n))
        for i in range(n):
            if 'offline' in rt[i + 1]:
                break
            nPos = rt[i + 1].index("\t")
            ds[i] = rt[i + 1][:nPos]
            dev = ds[i]
            # # 获取手机型号
            model = Test().get_device_name_with_IMEI(dev,'model')
            # 获取手机名称
            name = Test().get_device_name_with_IMEI(dev,'brand')
            # 包关键字
            package_keyword = keyword
            if len(package_keyword) <=0 :
                package_keyword = None
            # 所有包名
            pk = Test().get_package_name_contain(dev,package_keyword)
            if model is None or name is None:
                break
            mobile_folder_name = name + '_' + model + '_' + dev
            mobile_data[mobile_folder_name] = pk

        self.set_config_with_name('mobile',mobile_data)

    def get_all_devices(self,keyword):
        self.get_link_mobile_with_keyword(keyword)
        mobile = self.get_config_with_name('mobile')
        return mobile

    def get_config_with_name(self,name=None):
        if self.workspace is None:
            self.setup_config_file()
        config_file = self.workspace + self.config_name
        with open(config_file,'r') as f:
            data = json.load(f)
            if name is None:
                return data
            return data.get(name)

    def set_config_with_name(self,name,value):

        config_file = self.workspace  + self.config_name
        if config_file is None:
            self.setup_config_file()
        f = open(config_file,'r')
        data = json.load(f)
        f.close()
        with open(config_file,'w') as fw:
            data[name] = value
            json.dump(data,fw)

    def setup_config_file(self):
        if self.workspace is None:
            self.workspace =  self.get_workspace()

        config_path = self.workspace + '/' + self.config_name
        if  platform in ["Windows/Cygwin","Windows"]:
            config_path = config_path.replace('/','\\')
        if not os.path.exists(config_path):
            with open(config_path, 'w') as fw:
                config = {}
                config['workspace'] = self.workspace
                config['seed_num'] = self.seed_num
                config['time_delay'] = self.time_delay
                config['run_times'] = self.run_times
                config['implement_way'] = self.implement_way
                config['log_level'] = self.log_level
                config['life_cycle'] = self.life_cycle
                config['packages_testing'] = self.packages_testing
                config['cycles'] = self.cycles
                config['device'] = []
                json.dump(config,fw)


class FileManager:
    def start_logcat(self):
        pass



if __name__ == '__main__':

    # Test().start_test()
    TestUI().get_link_mobile_with_keyword('model')

    # dev = ['d934619c','46f1f44e']
    # for de in dev:
    #     adb_result = os.popen('adb -s ' + de + ' shell ps | grep monkey').readlines()
    #     #print(adb_result)
    #     for line in adb_result:
    #         pid = line.split(' ')[5]
    #         result =  os.popen('adb -s ' + de + ' shell kill ' + str(pid))
    #         #print(result)

    # IMEI = 'd934619c'
    # type = 'model'
    # name = ''
    # test = 'adb -s ' + IMEI + ' shell "cat /system/build.prop | grep "ro.product.' + type +'="" '
    # test = "adb -s " + IMEI + ' shell pm list packages | findstr ' + name
    # result = os.popen(test).readlines()
    #print(test)
    #print(result)

    


"""
Implementation of the command-line I{xyscript} tool.
"""
from __future__ import absolute_import

# For backward compatibility
__all__ = ['pull', 'pullsubmodule', 'initproject', 'package', "pps", 'syn', 'merge','diagnose','change','main']
from xyscript.api import pull, pullsubmodule, initproject, pps, package, syn, merge, main


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import getpass
import sys,os
from xyscript.common.xylog import faillog,warninglog

PW_NUM = 0
PW_ARRAY = []
SELECT_FILE_COUNT = 0

class Interaction:
    def get_user_input(self,des,password=False,pw_num=None):
        """
        获取用户输入
        des:      提示语，不加:
        password: 是否密文输入
        pw_num:   如果是密文，需要重复的次数
        """
        global PW_NUM,PW_ARRAY
        if pw_num == None:
            pw_num = 1
        pw_num = int(pw_num)
        
        if password == False:
            user_input = input(des + ":")
            return user_input
        else:
            pw = ""
            if PW_NUM == 0:
                pw = getpass.getpass(des + ":")
            else:
                pw = getpass.getpass(des + " again:")
            PW_ARRAY.append(pw)
            PW_NUM = PW_NUM +1
            if PW_NUM != pw_num :
                result =  self.get_user_input(des,password,pw_num)
                return result
            else:
                result_array = PW_ARRAY
                PW_NUM = 0
                PW_ARRAY = []
                if len(set(result_array)) == 1:
                    return result_array[0]
                else:
                    return None

    def select_directory(self,count=None):
        """
        选择一个文件夹
        count:允许用户取消的次数,默认是2次
        """
        global SELECT_FILE_COUNT
        if count == None:
            count = 2

        import tkinter as tk
        from tkinter import filedialog
        
        root = tk.Tk()
        root.withdraw()
        root.update()
        file_path = filedialog.askdirectory(initialdir=os.getcwd(), title="选择一个文件夹")

        if SELECT_FILE_COUNT >= int(count):
            faillog("you have give up choosing an folder " + str(count) + " times!") 
            return None

        if file_path == "":
            SELECT_FILE_COUNT = SELECT_FILE_COUNT + 1
            # print(SELECT_FILE_COUNT)
            warninglog("please choose an folder")
            result = self.select_directory(count=count)
            return(result)
        else:
            root.update()
            SELECT_FILE_COUNT = 0
            # root.mainloop()
            return file_path

if __name__ == "__main__":
#   print(Interaction().get_user_input("please enter password"))
#   print(Interaction().get_user_input("please enter password",password=True,pw_num=2))
    print(Interaction().select_directory(count=3))


#!/usr/bin/env python
# coding=utf-8
# -*- encoding:utf-8 -*-
# from __future__ import print_function

import json

import os
import sys
import stat

from git import Repo

from xyscript.common.xylog import *
import xyscript.common.globalvar as gv

project_name = ''
branch_name = ''
platform = ''

project_repo = None
have_remove_pod_lockfile = False

PROJECTS = [
    {'name': "driver", 'url': ""},
    {'name': "passenger",'url': ""},
    {'name': "test", 'url': "git@github.com:littertwo/iosTestDemo.git"}
]


def package_project_with_enter(projects):
    global project_name, branch_name, platform
    url = ''
    for index, project in enumerate(projects):
        if project['name'] == project_name:
            # 拉取项目
            package_project(project)
            return

    if url == '':
        print("没有找到项目名为：", project_name, "的项目")

# def select_project_to_package(projects=None):
#     project_count = len(projects)

#     if project_count > 0 :
#         print("发现",project_count,"个可打包工程，请选择：")
#         for index ,project_objc in enumerate(projects):
#             print(index+1,")",project_objc['name'])
#         print(project_count+1,"）above all")

#         project_index = input("请输入要打包的工程:")
#         package_project_with_index(project_index,projects)
#     else:
#         print("当前不存在可以打包的项目")

# def package_project_with_index(project_index,projects):
#     try:
#         index = int(project_index)
#         # 索引值正确
#         if index >0 and index<len(projects)+1:
#             package_projectlist([projects[index-1]])
#         elif index == len(projects)+1:
#             package_projectlist(projects)
#         else:
#             reinput_index = input("索引值不存在，请重新输入:")
#             package_project_with_index(reinput_index,projects)
#         # print(index)
#     except ValueError:
#         reinput_index = input("索引值应该为数字，请重新输入:")
#         package_project_with_index(reinput_index,projects)


def package_projectlist(project_list):
    for project in project_list:
        package_project(project)


def package_project(project):
    print("开始打包名为：", project['name'], "的项目...")
    # 创建目录
    try:
        os.mkdir(project['name'])
        pass
    except IOError as error:
        print(format(error))
    else:
        # print(os.getcwd())
        pass
        project_path = os.getcwd() + "/" + project['name']
        print(project_path)
        gitLabTool = GitLabTool(project_path)
        repo = gitLabTool.clone_project(project['url'], project['name'])
        # gitLabTool.change_branch(repo)
        # print(os.getcwd())
        os.chdir(project_path)
        # 执行拉取子模块脚本
        project_tool = IOSProjectTool()
        project_tool.run_script('ProjConfig.py')
        project_tool.run_pod_install()
        project_tool.fastlane_to_package()


class IOSProjectTool:
    def run_script(self, path):
        print("执行脚本，路径是", path)
        shell_str = "python3 " + path
        print(os.system(shell_str))

    def have_podfile(self):
        filepath = os.getcwd() + '/Podfile'
        if os.path.exists(filepath):
            return True
        return False

    def run_pod_install(self,script=None,ispackage=None):
        global have_remove_pod_lockfile
        if script == None:
            script = "pod install"
        print("start run " + script)

        try:
            # r = os.popen(script)
            # r = Log().printandresult(script)
            text = printandresult(script)
            # print(text)
            result = "Pod installation complete!" not in text
            if result:
                faillog("run " + script +" failed")
                if script == "pod install" and not have_remove_pod_lockfile: 
                    self.run_pod_install("pod install --repo-update")
                elif not have_remove_pod_lockfile:
                    os.system("rm Podfile.lock")
                    have_remove_pod_lockfile = True
                    self.run_pod_install("pod install")
                else:
                    if ispackage:
                        pass
                        # Email(gv.get_value("NOTIFICATION_EMAILS")).send_package_email(False)
                    sys.exit()
            # r.close()
            
        except BaseException:
            faillog("run " + script +" failed")
            if ispackage:
                pass
                # Email(gv.get_value("NOTIFICATION_EMAILS")).send_package_email(False)
            sys.exit()

    def run_cert_syn(self):
        print("项目执行fastlane syn")
        shell_str = "fastlane syn"
        print(os.system(shell_str))

    def fastlane_to_package(self):
        global platform
        print("fastlane 自动打包")
        shell_str = "fastlane package" + platform
        print(os.system(shell_str))


class GitLabTool:
    # def __init__(self, local_path=None):
    #     # repo = Repo("git@github.com:littertwo/iosTestDemo.git")
    #     # self.repo = repo
    #     pass

    def test(self):
        print("test")

    # 克隆项目
    def clone_project(self, url, local_path):
        global project_repo
        try:
            # 更改权限
            os.chmod('./', stat.S_IRWXU)
            # 克隆仓库
            project_repo = Repo.clone_from(
                url=url, to_path=local_path, progress=None)
            # print(project_repo.create_head('branchname'))
        except IOError as error:
            print(format(error))
        else:
            print("克隆项目成功")
            return project_repo

    def pull_submodule(self):
        try:
            currentPath = os.getcwd()
            repo = Repo(currentPath)
            initFlag = 0
            file = open("ProjConfig.json")
            moduleConfigList = json.load(file)
            for moduleConfig in moduleConfigList:
                module_path = currentPath + "/" + moduleConfig["module"]
                if not os.listdir(module_path):
                    initFlag = 1
            if initFlag == 1:
                print("submodule init...")
                repo.git.submodule('update', '--init')
                print("submodule init success")
            for moduleConfig in moduleConfigList:
                module_path = currentPath + "/" + moduleConfig["module"]
                sub_repo = Repo(module_path)
                sub_remote = sub_repo.remote()
                sub_repo.git.reset('--hard')
                sub_repo.git.checkout(moduleConfig["branch"])
                sub_remote.pull()
                print(module_path + " " +
                      moduleConfig["branch"] + " pull success")
        except BaseException as error:
            print("拉取子模块失败", error)

    # 切换分支
    def change_branch_g(self, branch_name, git=None):
        if git is None:
            repo = Repo(os.getcwd())
            # print("branch_name:",branch_name)
            local_branch_names = []#本地库列表
            remote_branch_names = []#远端库列表
            current_branch = repo.active_branch.name
            # print("current_branch_name:", current_branch)
            for localitem in repo.heads :
                local_branch_names.append(localitem.name)
                # print(localitem.name)
            # print("local_branch_names:", local_branch_names)

            for remoteitem in repo.refs:
                remote_branch_names.append(remoteitem.name)
                # print(remoteitem)
            # print("remote_branch_names:", remote_branch_names)

            if branch_name == current_branch:
                pass
                # print("就是当前分支，不需要切换")
            else:
                if branch_name in local_branch_names:
                    # print("本地存在目标分支，切换即可")
                    try:
                        local_target_branch = None
                        if branch_name == "develop":
                            local_target_branch = repo.heads.develop
                        elif branch_name == "Develop" :
                            local_target_branch = repo.heads.Develop
                        elif branch_name == "master" :
                            local_target_branch = repo.heads.master

                        repo.head.reference = local_target_branch
                    except BaseException as error:
                        errstr = "\033[1;31m" + "切换分支失败：" + str(error) + "\033[0m"
                        print(errstr)
                        sys.exit()
                    
                else:
                    remote_branch_name = 'origin/' + branch_name
                    if remote_branch_name in remote_branch_names:
                        # print("远端存在目标分支同名分支，checkout")
                        try:
                            git = repo.git
                            git.checkout('-b', branch_name, remote_branch_name)
                            # repo.remote().pull()
                            # print("切换分支成功")
                        except BaseException as error:
                            errstr = "\033[1;31m" + "checkout分支失败：" + str(error) + "\033[0m"
                            print(errstr)
                    else:
                        errstr = "\033[1;31m" + "远端不存在名为："+ branch_name + "的分支，无法checkout,请先创建远端仓库分支再checkout" + "\033[0m"
                        print(errstr)

        else:
            print(branch_name)

    def get_all_projects(self):
        print(PROJECTS)


# parm1：项目名称 parm2：分支名称 parm3：打包的平台
def main(argv):
    global project_name, branch_name, platform

    project_name = argv[1]
    branch_name = argv[2]
    platform = argv[3]

    print('开始打包....')

    # gitLabTool = GitLabTool("")
    # projects = gitLabTool.get_all_projects()
    # select_project_to_package(projects)
    package_project_with_enter(PROJECTS)


if __name__ == "__main__":
    print('开始执行脚本.....')
    # main(sys.argv)
    # print(IOSProjectTool()._have_podfile())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os,sys
from xyscript.packagePB.brew_package import *
from xyscript.packagePB.pypi_package import *
from xyscript.common.xylog import warninglog,faillog

__all__ = ['pypi','brew','all']

def do():
        pass

def main():
    platform = sys.argv[1]
    
    if platform not in __all__:
        faillog("please an platform in pypi,brew and all!")
        sys.exit()
    elif platform == 'pypi':
        upload_to_pypi()
    elif platform == 'brew':
        upload_to_brew()
    elif platform == 'all':
        upload_to_pypi()
        upload_to_brew()

if __name__ == '__main__':
    main()
    


#!/usr/bin/python
# -*- coding: UTF-8 -*-
 
import smtplib
from email import encoders
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart 
from email.mime.image import MIMEImage 
from email.header import Header
from email.mime.base import MIMEBase
from email.utils import parseaddr,formataddr
from xyscript.source.html import diagnosemail_body, diagnosemail_header,mergemail_body
from xyscript.source.image import jsonfileicon
import base64,os

# mail_host = "smtp.163.com"  # 设置服务器
# mail_user = "idouko@163.com"  # 用户名
# mail_pass = "XYCoder02"  # 三方邮箱口令
# sender = 'idouko@163.com'# 发送者邮箱

mail_host = "partner.outlook.cn"
mail_user = "cm@saicmobility.com"
mail_pass = "Saic123456"  # 三方邮箱口令
sender = 'cm@saicmobility.com'# 发送者邮箱

receivers = ['m18221031340@163.com']  # 接收邮件，可设置为你的QQ邮箱或者其他邮箱

class Email():
    def __init__(self,receiver=None):
        global receivers
        # print(receiver)
        if receiver is not None and len(receiver) > 0:
            receivers = receiver
        # print (receivers)
        # print(receivers)

    def send_package_email(self,success,url=None,image=None):
        global receivers
        string = ""
        if success:
            string = "您好！\n      项目打包成功，详情如下：\n 项目地址：%s\n打包分支：%s\n打包平台：%s\n打包网络环境：%s\n版本号：%s\n编译号：%s\n" %(address,branch,platform,env,version,build)
        else:
            string = "您好！\n      项目打包失败，请注意查看错误日志！信息如下：\n项目地址：%s\n打包分支：%s\n打包平台：%s\n打包网络环境：%s\n版本号：%s\n编译号：%s\n" %(address,branch,platform,env,version,build)
        Email(receivers).sendemail(string,"此邮件来自自动化打包","iOS项目组",image='/Users/v-sunweiwei/Downloads/timg.jpeg')

    def send_diagnose(self,git_objc=None):
        global receivers
        
        content_string = diagnosemail_body.format(project_url=git_objc['project_url']
                                                ,project_name=git_objc['project_name']
                                                ,user_name=git_objc['user_name']
                                                ,branch_name=git_objc['branch_name']
                                                ,commit_content=git_objc['commit_content']
                                                ,commit_url=git_objc['commit_url']
                                                ,commit_id=git_objc['commit_id']
                                                ,date=git_objc['date']
                                                ,result=git_objc['result']
                                                ,result_detail=git_objc['result_detail'])
        content = "<!DOCTYPE html><html lang=\"en\">" +  diagnosemail_header + content_string + "</html>"
        content = content.replace('\n','').encode("utf-8")

        image_name = 'jsonfileicon' + '.png'
        tmp = open(image_name, 'wb')
        tmp.write(base64.b64decode(jsonfileicon))
        tmp.close()
        # 收件人
        if 'user_email' in git_objc:
            receivers = git_objc['user_email']
        # 抄送人    
        cc = None
        if git_objc['cc'] is not None:
            cc = git_objc['cc']

        Email(receivers).sendemail(None,"此邮件来自代码自动诊断","iOS项目组",cc=cc,htmltext=content,image=image_name,filepath=git_objc['file_path'])
        os.remove(image_name)

    def send_merge_result(self,merge_result):
        global receivers
        
        content_string = mergemail_body.format(project_name=merge_result['project_name']
                                                ,package_branch=merge_result['package_branch']
                                                ,end_time=merge_result['end_time']
                                                ,start_time=merge_result['start_time'])
        # 壳工程
        shell_array = merge_result['shell']
        submodule_array = merge_result['submodule']
        private_modules = merge_result['private_module']
        shell_dom = ''
        for shell in shell_array:
            # 此处jiraurl需要用替换bug号
            bug_list = '&nbsp;'
            if 'jiraurl' in merge_result.keys():
                bug_list = self.commit_is_a_bug(shell['info'],merge_result['jiraurl'])
            
            shell_dom = shell_dom + '<tr><td>'+ shell['user'] +'</td><td>'+ shell['date'] +'</td><td>'+ shell['info'] +'</td><td>' + bug_list + '</td></tr>'
            # print(shell_dom)
        content_string = content_string.replace('<&shell_list&>',shell_dom)

        # 子模块
        sub_dom = ''
        for sub in submodule_array:
            sub_dom = sub_dom + '<H3>' + sub['name'] + '</H3>'
            sub_commit_dom = '<tr class="table-header" style="background-color: gray"><td>提交人</td><td>提交时间</td><td>提交内容</td><td>备注</td></tr>'
            for sub_commit in sub['commit_log']:
                sub_commit_link = '&nbsp;'
                if 'jiraurl' in merge_result.keys():
                    sub_commit_link = self.commit_is_a_bug(sub_commit['info'],merge_result['jiraurl'])
                sub_commit_dom = sub_commit_dom + '<tr><td>'+ sub_commit['user'] +'</td><td>' + sub_commit['date'] + '</td><td>' + sub_commit['info'] + '</td><td>' + sub_commit_link + '</td></tr>'
            sub_commit_dom = '<table class="result_table_shell" >' + sub_commit_dom + '</table>'
            sub_dom = sub_dom + sub_commit_dom
            # print(sub_dom)
        content_string = content_string.replace('<&submodule_list&>',sub_dom)

        # 共享库 private_module
        private_dom = ''
        for sub in private_modules:
            private_dom = sub_dom + '<H3>' + sub['name'] + '</H3>'
            sub_commit_dom = '<tr class="table-header" style="background-color: gray"><td>提交人</td><td>提交时间</td><td>提交内容</td><td>备注</td></tr>'
            for sub_commit in sub['commit_log']:
                sub_commit_link = '&nbsp;'
                if 'jiraurl' in merge_result.keys():
                    sub_commit_link = self.commit_is_a_bug(sub_commit['info'],merge_result['jiraurl'])
                sub_commit_dom = sub_commit_dom + '<tr><td>'+ sub_commit['user'] +'</td><td>' + sub_commit['date'] + '</td><td>' + sub_commit['info'] + '</td><td>' + sub_commit_link + '</td></tr>'
            sub_commit_dom = '<table class="result_table_shell" >' + sub_commit_dom + '</table>'
            private_dom = private_dom + sub_commit_dom
            # print(sub_dom)
        content_string = content_string.replace('<&private_modules&>',private_dom)
        
        
        content = "<!DOCTYPE html><html lang=\"en\">" +  diagnosemail_header + content_string + "</html>"
        # print(content)
        content = content.replace('\n','').encode("utf-8")

        cc = None
        if merge_result['cc'] is not None:
            cc = merge_result['cc']

        Email(receivers).sendemail(None,"此邮件来自代码合并","iOS项目组",cc=cc,htmltext=content)

    def commit_is_a_bug(self,commit,url):
        if commit is None or len(commit) <= 0 :
            return None
        
        commit_array = commit.split(' ')

        bug_list_array = []
        for commit_item in commit_array:
            item_array = commit_item.split('_')
            if len(item_array)>=3 and item_array[0] == 'T' and '-' in item_array[1]:
                url_filter = url.replace('',item_array[1])
                link_string = '<a href="' + url_filter + '" style="word-wrap:break-word;">' + item_array[1] + '</a>'
                bug_list_array.append(link_string)

        return ''.join(bug_list_array)
        

    def get_html_text(self,html_path):
        with open(html_path,'r') as f:
            # print(f.read())
            content = f.read()
            self.sendemail(None,"此邮件来自代码自动诊断","iOS项目组",htmltext=content,image='/Users/v-sunweiwei/Desktop//xyscript/xyscript/source/img/jsonfileicon.png',filepath='/Users/v-sunweiwei/Desktop/extension/人力图.jpg')


    def sendemail(self, content ,subject, form_name, cc=None ,htmltext=None ,image=None , filepath=None):
        """
        发送邮件
        content 正文文本
        subject 副标题
        form_name 邮件来源文本
        cc
        htmltext 网页
        image 图片
        filepath 附件
        """
        global receivers
        
        subject = subject#邮件来源
        #构建信息体
        message = MIMEMultipart('alternative') 
        
        #下面的主题，发件人，收件人，日期是显示在邮件页面上的。
        message['From'] = formataddr([form_name, sender])
        message['To'] = ";".join(receivers)
        message['Subject'] = Header(subject, 'utf-8')#编码方式
        if cc != None:
            message["Cc"] = cc
            receivers = receivers + cc.split(';')
            print(receivers)

        #构造文字内容   
        text = content    
        text_plain = MIMEText(text,'plain', 'utf-8')    
        message.attach(text_plain)    

        if image != None:
            #构造图片链接
            sendimagefile=open(image,'rb').read()
            image = MIMEImage(sendimagefile)
            image.add_header('Content-ID','<image1>')
            image["Content-Disposition"] = 'attachment; filename="testimage.png"'
            message.attach(image)

        if htmltext != None:
            #构造html
            #发送正文中的图片:由于包含未被许可的信息，网易邮箱定义为垃圾邮件，报554 DT:SPM ：<p><img src="cid:image1"></p>
            message.attach(MIMEText(htmltext,'html','utf-8')) 


        if filepath != None:
            #构造附件
            sendfile=open(filepath,'rb').read()
            text_att = MIMEText(sendfile, 'base64', 'utf-8') 
            text_att["Content-Type"] = 'application/octet-stream'  
            #以下附件可以重命名成aaa.txt  
            file_name = (filepath.split("/")[-1])
            text_att["Content-Disposition"] = 'attachment; filename="%s"' %(file_name)
            text_att.add_header('Content-ID','<file1>')
            #另一种实现方式
            # text_att.add_header('Content-Disposition', 'attachment', filename='aaa.txt')
            #以下中文测试不ok
            #text_att["Content-Disposition"] = u'attachment; filename="中文附件.txt"'.decode('utf-8')
            message.attach(text_att)  

        try:
            # 163 邮箱（25）
            # smtpObj = smtplib.SMTP()
            # smtpObj.connect(mail_host, 25)  # 25 为 SMTP 端口号
            # smtpObj.login(mail_user, mail_pass)
            # smtpObj.sendmail(sender, receivers, message.as_string())
            # print("邮件发送成功")
            # smtpObj.quit()

            # outlook邮箱 (587)
            smtpObj = smtplib.SMTP(mail_host, 587)
            smtpObj.starttls()
            smtpObj.login(mail_user, mail_pass)
            smtpObj.sendmail(sender, receivers, message.as_string())
            print("邮件发送成功")
            smtpObj.quit()
        except smtplib.SMTPException as e:
            print("无法发送邮件,原因是:" + str(e))

if __name__ == "__main__":
    # Email('m18221031340@163.com').send_package_email(False)
    # Email('m18221031340@163.com').send_diagnose('/Users/v-sunweiwei/Desktop//iosTestDemo/build/reports/report.json','content')
    # Email(['m18221031340@163.com'])
    # Email().get_html_text('/Users/v-sunweiwei/Desktop//xyscript/xyscript/source/htmlx/diagnosemail.html')
    # Email().sendemail('这是一封来自财务的报表邮件','你','财务部',cc='m13451923928@163.com,786857294@qq.com',htmltext='')
    # Email().send_diagnose()

    # print(Email().commit_is_a_bug('T_ASR-14997_【ios】【司机端&乘客端】【IM】【ATE】发送超长文案时切换到发送语音，格式错乱，切换回文字后，键盘盖住文案 T_ASR-14992_【ios】【司机端&乘客端】【IM】【ATE】IM发送图片手动放大查看，图片比例失调','http://jira..com:8080/browse/?filter=-1'))

    Email().sendemail('工资明细','ios','weiwei.sun@hand-china.com','m18221031340@163.com,m13365169690@163.com,786857294@qq.com')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import os
import sys
import stat
import time
import threading
import time
from git import Repo
import shutil

from xyscript.common.xylog import *
from xyscript.ios.cert import Cert
from xyscript.ios.CommonScript import IOSProjectTool
import xyscript.common.globalvar as gv
from xyscript.ios.iosProjectTool import IOSTool
from xyscript.config import Config
from xyscript.common.file import File
from xyscript.ios.gitandjira import GitLab

SELECT_FILE_COUNT = 0
WORK_SPACE = None
ENV_CHANGE_COUNT = 0
UPLOAD_TESTFLIGHT_COUNT = 1


class Package:
    
    def clone_form_address(self,address,local_path,project_folder_name):

        from pathlib import Path
        folder_name = ""
        if project_folder_name == None:
            folder_name = (address.split("/")[-1]).split(".")[0] +'_' + time.strftime('%Y-%m-%d_%H:%M:%S',time.localtime(time.time()))
        else:
            folder_name = project_folder_name
        # folder_name = (address.split("/")[-1]).split(".")[0] +'_' + time.strftime('%Y-%m-%d_%H:%M:%S',time.localtime(time.time()))

        try:
            #创建目录
            local_path = local_path + "/" + folder_name
            if Path(local_path).exists():
                warninglog(folder_name + "is exists")
            else:
                os.mkdir(local_path)
            
            try:
                os.chdir(local_path)
                warninglog("current workspace is: " + local_path)
            except BaseException as error:
                faillog("change worksapce failed")
                # Email(gv.get_value("NOTIFICATION_EMAILS")).send_package_email(False)
                sys.exit()
                
            # 更改权限
            os.chmod(local_path, stat.S_IRWXU)
            local_path = local_path + "/"
            Repo.clone_from(url=address, to_path=local_path, progress=None)
            successlog("clone project success")
        except BaseException as error:
            warninglog(format(error))

    def change_branch(self, branch_name, git=None):
        """
        change_branch
        """
        if git is None:
            repo = Repo(os.getcwd())
            # 更新远端分支列表
            repo.git.remote('update','origin','--prune')
            # print(os.getcwd())
            print("start change branch to:",branch_name)
            local_branch_names = []#本地库列表
            remote_branch_names = []#远端库列表
            current_branch = repo.active_branch.name
            # print("current_branch_name:", current_branch)
            for localitem in repo.heads :
                local_branch_names.append(localitem.name)
                # print(localitem.name)
            # print("local_branch_names:", local_branch_names)

            for remoteitem in repo.refs:
                remote_branch_names.append(remoteitem.name)
                # print(remoteitem)
            # print("remote_branch_names:", remote_branch_names)

            if branch_name == current_branch:
                # print("就是当前分支，不需要切换")
                warningstring = "current branch is already :" + branch_name
                warninglog(warningstring)
            else:
                if branch_name in local_branch_names:
                    # print("本地存在目标分支，切换即可")
                    try:
                        local_target_branch = None
                        if branch_name == "develop":
                            local_target_branch = repo.heads.develop
                        elif branch_name == "Develop" :
                            local_target_branch = repo.heads.Develop
                        elif branch_name == "master" :
                            local_target_branch = repo.heads.master
                        else:
                            local_target_branch = repo.heads[branch_name]

                        repo.head.reference = local_target_branch
                        successlog("change branch success")
                    except BaseException as error:
                        errstr = "change branch failed:" + str(error)
                        faillog(errstr)
                        # Email(gv.get_value("NOTIFICATION_EMAILS")).send_package_email(False)
                        sys.exit()
                    
                else:
                    remote_branch_name = 'origin/' + branch_name
                    if remote_branch_name in remote_branch_names:
                        # print("远端存在目标分支同名分支，checkout")
                        try:
                            git = repo.git
                            git.checkout('-b', branch_name, remote_branch_name)
                            # repo.remote().pull()
                            successlog("change branch success")
                        except BaseException as error:
                            errstr = "checkout failed:" + str(error)
                            faillog(errstr)
                    else:
                        show_tag = repo.git.show(branch_name)
                        if 'commit' in show_tag and 'Author:' in show_tag and 'Date:' in show_tag:
                            # 不是分支，是tag
                            try:
                                printandresult('git fetch origin tag ' + branch_name)
                                successlog("change tag success")
                            except BaseException  as error:
                                errstr = "checkout tag failed:" + str(error)
                                faillog(errstr)
                        else:
                            errstr = "have no branch named:" + branch_name + " exist,cannot to checkout"
                            # errstr = "\033[1;31m" + "远端不存在名为："+ branch_name + "的分支，无法checkout,请先创建远端仓库分支再checkout" + "\033[0m"
                            faillog(errstr)
                            # Email(gv.get_value("NOTIFICATION_EMAILS")).send_package_email(False)
                            sys.exit()

        else:
            print(branch_name)

    def pull_submodule(self,ignore_assets=False):
        """
        pull submodules
        """
        current_submodule = None
        try:
            currentPath = os.getcwd()
            repo = Repo(currentPath)
            initFlag = 0
            file = open("ProjConfig.json")
            moduleConfigList = json.load(file)
            rn_module_path = None
            for moduleConfig in moduleConfigList:
                module_path = currentPath + "/" + moduleConfig["module"]
                if moduleConfig["module"] == "submodules/module-rn":
                    rn_module_path = module_path
                if not os.listdir(module_path):
                    initFlag = 1
            if initFlag == 1:
                print("submodule init...")
                repo.git.submodule('update', '--init')
                successlog("submodule init success")
            copy_assets_path = os.getcwd() + '/' + 'image.json'
            have_copy_assets_sh = os.path.exists(copy_assets_path)
            for moduleConfig in moduleConfigList:
                current_submodule = moduleConfig["module"]
                module_path = currentPath + "/" + moduleConfig["module"]
                sub_repo = Repo(module_path)
                sub_remote = sub_repo.remote()
                sub_repo.git.remote('update','origin','--prune')
                sub_repo.git.reset('--hard')

                branch = ""
                bracnh_name = ""
                if "branch" in moduleConfig.keys():
                    branch = "\tbranch"
                    bracnh_name = "\t" + moduleConfig["branch"]
                    sub_repo.git.checkout(moduleConfig["branch"])
                elif "commit" in moduleConfig.keys():
                    branch = "\tcommit"
                    bracnh_name = "\t" + moduleConfig["commit"]
                    sub_repo.git.checkout(moduleConfig["commit"])
                elif "tag" in moduleConfig.keys():
                    branch = "\ttag"
                    bracnh_name = "\t" + moduleConfig["tag"]
                    sub_repo.git.checkout(moduleConfig["tag"])
                ignore_assets_path = self.is_key_in_json(current_submodule,copy_assets_path)
                if ignore_assets and have_copy_assets_sh and (ignore_assets_path is not None):
                    
                    sub_repo.git.clean('-df',ignore_assets_path)
                    sub_remote.pull()
                    print('已忽略资源文件夹：' + ignore_assets_path)
                else:
                    sub_remote.pull()
                string = "\t" + moduleConfig["module"] + logitem().successitem(bracnh_name) + " of " + logitem().successitem(branch) + " \tpull success"
                print(string)

                # 针对rn模块
            if rn_module_path is not None:
                warninglog("start setup rn module...")
                self.check_node_env()
                # 切换到rn模块目录
                os.chdir(rn_module_path)
                rn_ios_shell_path = rn_module_path + '/rn_ios'
                if os.path.isfile(rn_ios_shell_path):
                    printandresult(rn_ios_shell_path)
                else:
                    printandresult("npm install")
                    printandresult('npm install scheduler  --save')
                    printandresult("rm -rf bundle")
                    printandresult("mkdir bundle")
                    printandresult("node node_modules/react-native/local-cli/cli.js bundle --entry-file index.js  --platform ios --dev false --bundle-output bundle/index.jsbundle --assets-dest bundle")
                successlog("setup rn module success!")
                # 切回原目录
                os.chdir(currentPath)
                successlog("restore workspace success!")
        except BaseException as error:
            if current_submodule is not None:
                submodule_error = "pull " + current_submodule + " failed"
                faillog(submodule_error)
            errorstr = "pull submodule failed:" + format(error)
            faillog(errorstr)
            # sys.exit()
    def is_key_in_json(self,key,file):
        assets_path = None
        with open(file,'r',encoding='utf8')as fp:
            json_data = json.load(fp)
            assets_list = json_data['imageList']
            for assets_item in assets_list:
                if key in assets_item['image']:
                    path = assets_item['assets']
                    assets_path = path.split(key)[1][1:] + '/'
                    break
        return assets_path

    def check_node_env(self):
        warninglog("start check node environment")
        result = printandresult('node -v')
        if 'node: command not found' in result:
            printandresult('brew install node')

    def run_extension_sh(self,file_list):
        for file in file_list:
            file_path = os.getcwd() + '/' + file
            if os.path.exists(file_path):
                warninglog("执行文件：" + file_path)
                printandresult("sh " + file_path)

    def change_default_net_env(self,env):
        global ENV_CHANGE_COUNT
        filepath = self._search_xcodeproj() + "/project.pbxproj"
        print("start change default environment variable")
        file_data = ""
        try:
            with open(filepath,encoding="utf-8") as f:
                for line in f:
                    if "SC_URL_TYPE =" in line:
                        # print(line,end="")
                        line = str.split(line,'SC_URL_TYPE')[0] + "SC_URL_TYPE = " + env +";\n"
                    file_data += line

            with open(filepath,"w",encoding="utf-8") as f:
                f.write(file_data)
            successlog("change " + env +" environment success")   
        except BaseException  as error:
            faillog("change " + env +" environment failed:" + format(error))
        
    def _just_for_uploadbugly(self):
        filepath = self._search_xcodeproj() + "/project.pbxproj"
        print("start add build setting for upload bugly")
        file_data = ""
        try:
            with open(filepath,encoding="utf-8") as f:
                for line in f:
                    add_line = ""
                    if "SC_URL_TYPE =" in line:
                        add_line = str.split(line,'SC_URL_TYPE')[0] + "SC_BUGLY_DEFAULT = default;\n"
                    file_data += line 
                    file_data += add_line
            with open(filepath,"w",encoding="utf-8") as f:
                f.write(file_data) 
            successlog("add build setting for upload bugly success")
        except BaseException  as error:
            faillog("add build setting for upload bugly failed:" + format(error))

        
    def _search_xcodeproj(self):
        paths = os.listdir(os.getcwd())
        for filename in paths:
            if ".xcodeproj" in filename :
                return filename
        faillog("The current path does not have a: xcodeproj file,such as :projectname.xcodeproj")
        # Email(gv.get_value("NOTIFICATION_EMAILS")).send_package_email(False)
        sys.exit()

    def _add_empty_file(self,fileName="package_empty_file"):
        f= open(fileName,"w")
        f.close
    
    def _pull_lastest_code(self,branch):
        currentPath = os.getcwd()
        repo = Repo(currentPath)
        repo.git.checkout(branch)
        file = open("ProjConfig.json")
        moduleConfigList = json.load(file)
        print("pull " + logitem().successitem("shell") + " in branch " + logitem().successitem(branch) + " success")
        repo.git.pull()
        for moduleConfig in moduleConfigList:
            module_path = currentPath + "/" + moduleConfig["module"]
            sub_repo = Repo(module_path)
            sub_repo.git.checkout(moduleConfig["branch"])
            sub_repo.git.pull()
            print("pull " + logitem().successitem(moduleConfig["module"]) + " in shell " + logitem().successitem(branch) + " success")

    def _get_current_config_file(self):
        file = open("ProjConfig.json")
        moduleConfigList = json.load(file)
        return moduleConfigList

    def _clear_local_workspace(self,date):
        currentPath = os.getcwd()
        repo = Repo(currentPath)
        repo.git.reset('--hard')#清空壳工程本地工作空间
        for moduleConfig in date:
            module_path = currentPath + "/" + moduleConfig["module"]
            sub_repo = Repo(module_path)
            sub_repo.git.reset('--hard')
        print(logitem().successitem("clear local workspace success"))

    def add_tag_after_package(self,tag):
        try:
            if tag is not None:
                currentPath = os.getcwd()
                repo = Repo(currentPath)
                if tag not in repo.git.tag():
                    repo.git.tag(tag)
                    repo.git.push('origin',tag)
                    successlog("add tag to shell success")
                    #子模块添加tag
                    currentPath = os.getcwd()
                    file = open("ProjConfig.json")
                    moduleConfigList = json.load(file)
                    for moduleConfig in moduleConfigList:
                        module_path = currentPath + "/" + moduleConfig["module"]
                        sub_module = Repo(module_path)
                        sub_module.git.tag(tag)
                        sub_module.git.push('origin',tag)
                        successlog("add tag to " + moduleConfig["module"] + " success")
                else:
                    warninglog("tag named: " + tag + " is exists!")
        except BaseException as error:
            faillog("add tag failed:" + format(error))

    def cache_symbol_at_home(self,path):
        #复制符号表到本地
        try:
            root_path = Config().get_sys_root_path()
            sym_path = "symbol/"
            cache_path = root_path + Config().get_config("File-store-path","cache_path") + sym_path + path
            File().creat_folder(cache_path)
            folder_name = os.getcwd()
            files = File().find_files_contain(folder_name,".app.dSYM.zip")
            if len(files) > 0:
                for file in files:
                    shutil.copy(file,cache_path)
                successlog("cache symbol files success")
            else:
                warninglog("have no symbol need to cache")
        except BaseException as error:
            faillog("cache symbol failed:" + format(error))

    def handle_package_order(self,order_array=None,project_version=None, project_build=None, env=None):
        failed_platform = []
        for index, platform_name in enumerate(order_array):
            shell_str = "fastlane package" + platform_name + project_version + project_build + env
            result = self._platform_package(shell_str)
            if index != len(order_array) -1:
                IOSTool().add_content_to_file("AppDelegate.h")
            if result == False:
                failed_platform.append(platform_name)
        
        if len(failed_platform) > 0:
            faillog("package to all failed")
        else:
            successlog("package to all success")

    def _platform_package(self,shell_str):
        global UPLOAD_TESTFLIGHT_COUNT
        text = printandresult(shell_str)
        if "fastlane.tools just saved" in text or "fastlane.tools finished successfully" in text:
            UPLOAD_TESTFLIGHT_COUNT = 1
            return True 
        elif (("An error occurred while uploading the package" in text) 
                or ("An error occurred while uploading the file" in text) 
                or ("Error uploading ipa file, for more information see above" in text) 
                or ("fastlane finished with errors" in text) 
                or ("Error building the application - see the log above" in text)):
            if UPLOAD_TESTFLIGHT_COUNT < 3:
                faillog("upload to testflight failed " + str(UPLOAD_TESTFLIGHT_COUNT) + " times")
                UPLOAD_TESTFLIGHT_COUNT = UPLOAD_TESTFLIGHT_COUNT +1
                warninglog("start uploading " + str(UPLOAD_TESTFLIGHT_COUNT) + " time")
                shell_str1 = shell_str
                if "packagetestflight2" not in shell_str:
                    shell_str1 = shell_str.replace("packagetestflight","packagetestflight2")
                return self._platform_package(shell_str1)
            else:
                UPLOAD_TESTFLIGHT_COUNT = 1
                return False
        else:
            UPLOAD_TESTFLIGHT_COUNT = 1
            return False


    def package_to_platform(self,platform=None, project_version=None, project_build=None, emails=None, env=None,package_order=None):
        """
        配置证书 等效于 fastlane + platform
        """
        #获取项目当前version、build

        if platform == None:
            platform = "pgyer"
            
        version_str = ""
        build_str = ""
        env_str = ""
        if project_version != None:
            version_str = " version:" + project_version
        
        if project_build != None:
            build_str = " build:" + project_build

        if env != None:
            env_str = " env:" + env

        
        print("start package to " + platform)
        if Cert().fastlane_is_in_gem() or Cert().fastlane_is_in_brew() :
            shell_str = "fastlane package" + platform + version_str + build_str + env_str
            try:
                if Cert()._have_fastfile():
                    if platform == "all":#分开打包防止多次打包结果相互影响

                        if package_order == None or package_order == "0":
                            self.handle_package_order(["testflight","pgyer"],version_str,build_str,env_str)
                        elif package_order == "1":
                            self.handle_package_order(["pgyer","testflight"],version_str,build_str,env_str)
                        else:
                            faillog("nonsupport package_order")
                    else:
                        result = self._platform_package(shell_str)
                        if result:
                            successlog("package to " + platform + " success")
                        else:
                            faillog("package to " + platform + " failed")

                else:
                    faillog("You may not have the fastfile,please set fastlane up first!")
                    sys.exit()
                
            except BaseException  as error:
                faillog(str(error))
                sys.exit()
        else:
            warninglog("You may not have the fastlane installed yet,autoinstall now...")
            Cert()._install_fastlane()
            self.package_to_platform(platform,project_version,project_build,env=env)
    

    def auto_package(self, project_address=None, branch_name=None, platform=None, net_env=None, project_version=None, project_build=None,emails=None,tag_name=None,package_order=None,package_folder=None):
        if branch_name == None:
            branch_name = "Develop"
        if platform == None:
            platform = "pgyer"
        if emails == None:
            emails = []

        print("自动打包\n项目：",project_address,"\n分支为：",branch_name,"\n发布平台为：",platform,"\n网络环境：",net_env,"\n版本号：",project_version,"\n编译号：",project_build)
        # print("please choose an folder")
        
        # clone
        # self.clone_form_address(project_address,self.select_directory())
        self.clone_form_address(project_address,os.getcwd(),package_folder)
        # 切换分支
        self.change_branch(branch_name)
        # 拉子模块
        self.pull_submodule()
        # pod indtall
        IOSProjectTool().run_pod_install()
        # 选择网络环境
        if net_env != None:
            self.change_default_net_env(net_env)
        # #fastlane syn
        # Cert().run_cert_syn()
        # #安装 pgyer 插件
        os.system("bundle install")
        # #打包
        self.package_to_platform(platform, project_version, project_build,emails,env=net_env,package_order=package_order)
        # 打tag
        self.add_tag_after_package(tag_name)
        # 缓存symbol
        path = os.getcwd().split("/")[-1]
        Package().cache_symbol_at_home(path)

    def merge_code(self,code_branch,package_branch,email,url=None,keyword=None):
        """
        合并代码(开发分支名、打包分支名)
        """
        if code_branch == None:
            code_branch = "Develop"
        if package_branch == None:
            package_branch = "zuche-test"

        try:
            self.change_branch(code_branch)
            currentPath = os.getcwd()
            repo = Repo(currentPath)
            moduleConfigList = self._get_current_config_file()
            # 清空本地工作空间
            self._clear_local_workspace(moduleConfigList)
            # 检出 code_branch 分支的最新代码
            self._pull_lastest_code(code_branch)
            codebranch_config = self._get_current_config_file()
            # 检出 package_branch 分支的最新代码
            self._pull_lastest_code(package_branch)
            # 合并 code_branch 分支到当前 package_branch 分支
            for moduleConfig in codebranch_config:
                module_path = currentPath + "/" + moduleConfig["module"]
                sub_repo = Repo(module_path)
                merge_str = "merge " + moduleConfig["branch"] + " into " + package_branch
                sub_repo.git.merge(moduleConfig["branch"],'-m', merge_str, '--log=10')
                print("merge " + logitem().successitem(moduleConfig["module"]) + " success")
            repo = Repo(currentPath)
            merge_str = "merge " + code_branch + " into " + package_branch
            repo.git.merge(code_branch, '-m', merge_str ,'--log=10')
            print("merge "+ logitem().successitem("shell") + " from " + logitem().successitem(code_branch) + " to " + logitem().successitem(package_branch) + " success")
            # repo.git.commit('-a', '-m', merge_str)

            #推远端
            repo.git.push()
            print("push " + logitem().successitem("shell") + " success")
            date = self._get_current_config_file()
            for moduleConfig in date:
                module_path = currentPath + "/" + moduleConfig["module"]
                sub_repo = Repo(module_path)
                sub_repo.git.push()
                print("push " + logitem().successitem(moduleConfig["module"]) + " success")
            
            # 还原到原始分支

            # 发送邮件给相关人员
            if email is not None:
                GitLab().get_all_commit_between_push(code_branch=code_branch,package_branch=package_branch,workspace=currentPath,mail=email,url=url,keyword=keyword)

        except BaseException as error:
            errorstr = "merge failed:" + format(error)
            faillog(errorstr)
            # sys.exit()


    def init_project(self, project_address=None, branch_name=None, platform=None, net_env=None):
        if branch_name == None:
            branch_name = "Develop"
        if platform == None:
            platform = "pgyer"

        print("initial 项目：",project_address," 分支为：",branch_name," 发布平台为：",platform,"网络环境：",net_env)
        print("please choose an folder")
        
        # #clone
        # self.clone_form_address(project_address,self.select_directory())
        # #切换分支
        self.change_branch(branch_name)
        # #拉子模块
        self.pull_submodule()
        # #pod indtall
        IOSProjectTool().run_pod_install()
        # #选择网络环境
        if net_env != None:
            self.change_default_net_env(net_env)
        # #fastlane syn
        Cert().run_cert_syn()

if __name__ == "__main__":
    # pass
    # Package().change_default_net_env("dev")
    # Package()._get_project_version()
    # Package().merge_code("Develop","zuche-test")
    # Package()._add_empty_file()
    # Package()._just_for_uploadbugly()
    # Package().pull_submodule()
    # IOSTool().add_content_to_file("AppDelegate.h")
    # Package().add_tag_after_package("tag_03")
    # path = os.getcwd().split("/")[-1]
    # Package().cache_symbol_at_home(path)
    # path = '/Users/v-sunweiwei/Desktop/saic/sam
    
    # print(Package().check_node_env())

    # path = '/Users/v-sunweiwei/Desktop/saic/ios-shell-passenger'
    # os.chdir(path)
    # repo = Repo(path)
    # printandresult('git ls-remote --tags origin')
    # printandresult('git show SRP_28722')
    # print('Author:' in repo.git.show('SRP_2872'))

    # Package().run_extension_sh(['branchS.sh'])
    # Package().is_key_in_json('submodules/module-login','/Users/sunweiwei/Desktop/Project/ios-shell-driver-cz/image.json')
     Package().pull_submodule(True)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os,sys,json
from xyscript.common.xylog import *
import webbrowser
from git import Repo
from xyscript.ios.package import Package
from xyscript.common.file import File
import re
from xyscript.common.mail import Email
from xyscript.ios.gitandjira import GitLab

class OCLint:
    def check_oclint(self):
        result = os.popen("which oclint")
        path = result.read()
        if len(path) <= 0:
            printandresult("brew tap oclint/formulae")
            printandresult("brew install oclint")
        else:
            successlog("the path of oclint is:" + path) 

    def check_xcpretty(self):
        result = os.popen("which xcpretty")
        path = result.read()
        if len(path) <= 0:
            printandresult("gem install xcpretty")
        else:
           successlog("the path of xcpretty is:" + path) 

    def run_oclint(self,git_objc=None):
        # try:
            
        self.check_oclint()
        self.check_xcpretty()
        path = os.getcwd()
        os.chdir(path)
        printandresult("xcodebuild clean")

        # printandresult("xcodebuild | tee xcodebuild.log")

        printandresult("xcodebuild | xcpretty -r json-compilation-database")
        printandresult("cp build/reports/compilation_db.json build/reports/compile_commands.json")
        reports_path = path + "/build/reports"
        reports_json_path = reports_path + "/compile_commands.json"

        with open(reports_json_path,'r') as json_file:
            if len(json_file.read()) == 2:
                faillog('输出文件为空，不需要反馈')
                sys.exit()

        os.chdir(reports_path)
        printandresult("oclint-json-compilation-database -e Pods \
                                                            -e DerivedSources \
                                                            -e '(.(HttpClient\.m))$' \
                                                            -- \
                                                            -disable-rule=LongLine \
                                                            -rc=LONG_VARIABLE_NAME=30 \
                                                            -rc=NCSS_METHOD=100 \
                                                            -max-priority-1=100000 \
                                                            -max-priority-2=100000 \
                                                            -max-priority-3=100000 \
                                                            >> report.json")
        # html_path = "file://" + reports_path + "/report.json"
        file_path = reports_path + "/report.json"

        if not os.path.exists(file_path):
            faillog('输出文件不存在')
            sys.exit()

        with open(file_path,'r') as json_file:
            if len(json_file.read()) <= 2:
                faillog('输出文件为空，不需要反馈')
                sys.exit()
        # webbrowser.open_new_tab(html_path)

        if git_objc is None:
            printandresult("open " + reports_path + "/report.json")
        else:
            mail_addresses = self._get_lastest_push_user_mail(path)
            # html_path = '/Users/v-sunweiwei/Desktop//ios-shell-driver/build/reports/report.json'
            push_item = self._get_push_info(path)
            diagnose_detail = self._get_diagnose_result_detail(file_path)
            diagnose = self._get_diagnose_result(diagnose_detail)

            html_path = self.get_result_html_file(file_path)
            print(html_path)

            diagnose_result = 'warning:' +  str(diagnose['warning']) + ' lines,' + 'OCLint Report:' + diagnose['report']

            content = "项目名称：%s\n提交人：%s\n提交时间：%s\n诊断结果为：%s\n详细说明见附件：" %(push_item['name'],push_item['user'],push_item['date'],diagnose_result)
            
            git_objc['project_name'] = push_item['name']
            git_objc['user_name'] = push_item['user']
            git_objc['date'] = push_item['date']
            git_objc['commit_content'] = push_item['commit_content']
            git_objc['result'] = diagnose_result
            git_objc['result_detail'] = diagnose_detail
            git_objc['file_path'] = html_path
            if 'user_email' in push_item:
                git_objc['user_email'] = push_item['user_email']

            current_branch_name = GitLab().get_current_branch_name(path)
            if current_branch_name is not None:
                git_objc['branch_name'] = current_branch_name
            
            Email(push_item['user_email']).send_diagnose(git_objc)

        # except BaseException as error:
        #     faillog("run diagnose failed:" + format(error))

    def get_result_html_file(self,file_path):
        try:
            
            # 格式化
            json_data = open(file_path, encoding='utf-8')
            html_body = ''
            for line in json_data.readlines():
                line_content = line.split('\n')[0]
                if 'Compiler Warnings:' in line_content or 'OCLint Report' in line_content or '[OCLint (http://oclint.org)' in line_content:
                    html_body = html_body + '<h4>' + line_content + '</h4>'
                elif len(line_content) == 0 or line_content == '\n':
                    pass
                elif 'Summary: TotalFiles=' in line_content :
                    html_body = html_body + '<h3>' + line_content + '</h3>' + '<br/>'
                else:
                    p = re.compile(r'(.((\d+):(\d+):).)')
                    r_result = p.search(line_content)
                    span = r_result.span()
                    error_path = line_content[0:int(span[0])]
                    error_line_num = line_content[int(span[0]):int(span[1])]
                    error_des = line_content[int(span[1]):]
                    html_body = html_body + "<p style='color: blue;display: inline;'>" + error_path + "</p>" + error_line_num + "<p style='color:red;display: inline;'>" + error_des + "</p>" + '<br/>'
            html_header = "<head> \
                            <title></title> \
                            <meta charset=\"UTF-8\"> \
                            <meta name=\"viewport\" content=\"width=device-width, initial-scale=1\"> \
                            </head>"
            html_body = "<!DOCTYPE html><html lang=\"en\">" + html_header + html_body + "</html>"
            html_path = '/'.join(file_path.split('/')[:-2]) + '/reports/report.html'
            # print(html_path)
            if os.path.exists(html_path):
                with open(html_path,'w+') as html_file:
                    # 清空文件
                    html_file.seek(0)
                    html_file.truncate()
            with open(html_path,'w+') as html_file:
                html_file.write(html_body)
            
            return html_path
        except BaseException as error:
            faillog('生成html文件失败:' + format(error))
            return None

    
    def _get_diagnose_result_detail(self,file_path):
        with open(file_path, encoding='utf-8') as file_obj:
            contents = file_obj.read()
            return contents

    def _get_diagnose_result(self,file_obj):
        result = {}
        warnings_array = [1,1,1]
        warnings = re.findall(r'Compiler Warnings\:((?:.|\n)*?OCLint Report)',file_obj)
        if warnings and len(warnings)>0 :
            warnings_array =  warnings[0].split('\n')
        result['warning'] = len(warnings_array) -3

        oclint_report = re.findall(r'Summary\:((?:.|\n)*?)\n',file_obj)
        if oclint_report and len(oclint_report)>0:
            result['report'] = str(oclint_report[0])[1:-1]
        else:
            result['report'] = ''
        return result
    
    def _get_push_info(self,workspace):
        push_item = {}
        repo = Repo(workspace)
        result =  repo.git.log('-1')
        # 项目名称
        project_name = (workspace.split("/")[-1]).split(".")[0]
        push_item['name'] = project_name

        # 提交用户名
        user = re.findall(r'Author\:((?:.|\n)*?)<',result)
        if user and len(user)>0:
            psuh_user = user[0]
            psuh_user = psuh_user.replace(' ','')
            push_item['user'] = psuh_user

        # 提交用户邮箱
        user_email = re.findall(r'Author\:((?:.|\n)*?)<(.*)>',result)
        if len(user_email) > 0:
            push_item['user_email'] = [user_email[0][1]]

        # 提交时间
        date = re.findall(r'Date:((?:.|\n)*?)\n',result)
        if date and len(date) > 0:
            push_date = date[0]
            push_date = push_date.replace('   ','')
            push_item['date'] = push_date

        # 提交信息
        commit_content = result.split('\n')[4:]
        push_item['commit_content'] = '\n'.join(commit_content)
        
        return push_item




    def _get_lastest_push_user_mail(self,workspace):
        repo = Repo(workspace)
        result =  repo.git.show('--stat')

        authors = []
        res = re.findall(r'Author\:((?:.|\n)*?>)',result)
        for author in res:
            res = re.findall(r'((?<=\<)[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,8}(?=\>))',author)
            for mail_address in res:
                authors.append(mail_address)
        # print(authors)
        return authors

    def setup_lint_for_package_machine(self):
        path = os.getcwd()
        os.chdir(path)
        if not os.path.exists(path + '/modules.json'):
            faillog("have no file named ‘modules.json’ in current workspace")
            sys.exit()

        with open(path + '/modules.json','r') as module_file:
            json_data = json.load(module_file)
            for package_name,packages_array in json_data.items():
                package_path = path + '/' + package_name
                File().creat_folder(package_path)
                os.chdir(package_path)
                for module_name,module_url in packages_array.items():
                    # Package().change_branch('zuche-Develop')
                    project_path = package_path + '/' + module_name
                    if not os.path.exists(project_path):
                        printandresult('git clone ' + module_url)
                    gitlab_ci_path = project_path + '/.gitlab-ci.yml'
                    os.chdir(project_path)
                    if package_name == 'zhuanche':
                        Package().change_branch('Develop')
                        self.add_yml_file(project_path,'-c test@test.com')
                    elif package_name == 'zuche':
                        Package().change_branch('zuche-Develop')
                        self.add_yml_file(project_path,'-c test@test.com')
                    elif package_name == 'public':
                        Package().change_branch('Develop')
                        self.add_yml_file(project_path,'-c test@test.com -c test@test.com')
                        Package().change_branch('zuche-Develop')
                        self.add_yml_file(project_path,'-c test@test.com -c test@test.com')
                    os.chdir(package_path)   
                os.chdir(path)

    def add_yml_file(self,path,cc):
        gitlab_ci_path = path + '/.gitlab-ci.yml'
        if os.path.exists(gitlab_ci_path):
            with open(gitlab_ci_path,'w+') as html_file:
            # 清空文件
                html_file.seek(0)
                html_file.truncate()
        # if not os.path.exists(gitlab_ci_path):
        ci_str = 'before_script:\n\
  - cd %s\n\
  - xyscript pull -b $CI_COMMIT_REF_NAME\n\
stages:\n\
  - build\n\
build_project:\n\
  stage: build\n\
  variables:\n\
      branch_url: $CI_PROJECT_URL/tree/$CI_COMMIT_REF_NAME\n\
      commit_url: $CI_PROJECT_URL/commit/$CI_COMMIT_SHA\n\
  script:\n\
    - xyscript diagnose -t $branch_url -b $CI_COMMIT_REF_NAME -a $commit_url -d $CI_COMMIT_SHA %s' % (path,cc)
        print(ci_str)
        with open(gitlab_ci_path,'w+',encoding="utf-8") as ci_file:
            ci_file.write(ci_str)
        
        repo = Repo(path)
        repo.git.add('--all')
        repo.git.commit('-m','setup ci file')
        repo.git.pull('origin')
        repo.git.push('origin')

def main():
    OCLint().run_oclint()
    # git_objc = {}
    # git_objc['project_url'] = 'https://www.baidu.com' 	
    # git_objc['branch_name'] = 'Develop' 
    # git_objc['commit_content'] = "test" 
    # git_objc['commit_url'] = 'commit_url'
    # git_objc['commit_id'] = 'commit_id'
    # git_objc['cc'] = 'm18221031340@163.com'
    # OCLint().run_oclint(git_objc)
    # OCLint()._get_diagnose_result('/Users/v-sunweiwei/Desktop//ios-shell-driver/build/reports/report.json')
    # print(OCLint().get_result_html_file('/Users/v-sunweiwei/Desktop//ios-shell-passenger/build/reports/report.json'))

    OCLint().setup_lint_for_package_machine()

if __name__ == "__main__":
    main()
    # p = re.compile(r'(.((\d+):(\d+):).)')
    # # line_content = '/Users/v-sunweiwei/Desktop//ios-shell-passenger/Pods/Headers/Public/SCStyle_libs/SCStyle.h:104:1: multiple declarations of method \'green\' found and ignored'
    # line_content = '/Users/v-sunweiwei/Desktop//ios-shell-passenger/Pods/Headers/Public/SCShare/SCSocialPlatformConfig.h:14:17: pointer is missing a nullability type specifier (_Nonnull, _Nullable, or _Null_unspecified)'
    # span = p.search(line_content)
    # print(span)
    # span = span.span()
    # error_path = line_content[0:int(span[0])]
    # error_des = line_content[int(span[1]):]
    # print(error_path)

    # print(error_des)

    # path = "/Users/v-sunweiwei/Desktop/saic/ios-shell-passenger"
    # repo = Repo(path)
    # print(repo.git.log('-1'))
    # OCLint()._get_push_info(path)
    # print(OCLint()._get_lastest_push_user_mail(path))

    # path = "/Users/v-sunweiwei/Desktop/saic/saic-ui/build/reports/compile_commands.json"
    # with open(path,'r') as file:
    #     if len(file.read()) == 2:
    #         print(123)




#-*- encoding:utf-8 -*-
import os, sys, json, re
from xyscript.common.xylog import*
import xyscript.common.globalvar as gv

class Cert:
    def run_cert_syn(self,ispackage=None):
        """
        拉取最新证书 等效于 fastlane syn
        """
        print("start pull lastest certs")
        if self.fastlane_is_in_gem() or self.fastlane_is_in_brew() :
            shell_str = "fastlane syn"
            try:
                if self._have_fastfile():
                    # os.system(shell_str)
                    # result = os.popen(shell_str)
                    # text = result.read()
                    text = printandresult(shell_str)
                    # print(text)
                    if "fastlane.tools finished successfully" in text:
                        successlog("pull latest certs success")
                    else:
                        faillog("pull latest certs failed")
                    
                    # result.close()
                else:
                    faillog("You may not have the fastfile,please set fastlane up first!")
                    if ispackage:
                        pass
                        # Email(gv.get_value("NOTIFICATION_EMAILS")).send_package_email(False)
                    sys.exit()
                
            except BaseException  as error:
                faillog(str(error))
                sys.exit()
        else:
            warninglog("You may not have the fastlane installed yet,autoinstall now...")
            self._install_fastlane()
            self.run_cert_syn(ispackage)
    
    def run_cert_pps(self):
        """
        配置证书 等效于 fastlane pps
        """
        print("start config certs")
        if self.fastlane_is_in_gem() or self.fastlane_is_in_brew() :
            shell_str = "fastlane pps"
            try:
                if self._have_fastfile():
                    os.system(shell_str)
                    successlog("config certs success")
                else:
                    faillog("You may not have the fastfile,please set fastlane up first!")
                    sys.exit()
                
            except BaseException  as error:
                faillog(str(error))
                sys.exit()
        else:
            warninglog("You may not have the fastlane installed yet,autoinstall now...")
            self._install_fastlane()
            self.run_cert_pps()

    def fastlane_is_in_gem(self):
        shell_str = "gem list"
        # r = os.popen(shell_str)
        # text = r.read()
        # r.close()
        text = printandresult(shell_str)
        # part = "fastlane\s\(\d{1,10}\.\d{1,10}\.\d{1,10}\.\)"
        # result = re.findall(text,part)
        # faillog(text)
        return "fastlane (" in text

    def fastlane_is_in_brew(self):
        shell_str = "brew list"
        # r = os.popen(shell_str)
        # text = r.read()
        # r.close()
        text = printandresult(shell_str)
        return "fastlane" in text
    
    def _have_fastfile(self):
        path = os.getcwd() + "/fastlane/Fastfile"
        return os.path.isfile(path)

    def _install_fastlane(self):
        try:
            #install xcode-select
            os.system("xcode-select --install")
            warninglog("start install fastlane,you may need to enter a password...")
            os.system("sudo gem install fastlane -NV")
        except BaseException as error:
            faillog(str(error))
            sys.exit()

    def pgyerplugin_is_in_gem(self):
        pass

        # gen_list = os.system(shell_str)
        # print(count(gen_list))
        # try:
        #     os.system(shell_str)
        # except BaseException as error:
        #     faillog(error)
       
        # successlog("run fastlane success")


if __name__ == "__main__":
    Cert().fastlane_is_in_gem()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# AUTHOR: XYCoder
# FILE: ~/Desktop/Project/xyscript/xyscript/crawler/version.py
# DATE: 2021/01/05 Tue
# TIME: 14:00:05

# DESCRIPTION:用于实时爬取pypi.org本库最新版本

import requests
from bs4 import    BeautifulSoup
url = 'https://pypi.org/project/xyscript/'

class Version:
    def get_last_version():
        strhtml = requests.get(url)
        soup = BeautifulSoup(strhtml.text,'lxml')
        data = soup.select('#history > div > div.release.release--latest.release--current > a > p.release__version')
        version_string = data[0].get_text()
        version_string = version_string.strip()
        # version_string = version_string.replace(' ', '')
        # print(version_string)
        return version_string

if __name__ == "__main__":
    Version.get_last_version()



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import os,sys,re
import platform
from xyscript.common.xylog import faillog,successlog,warninglog


class SysEvn():
    def get_system_environment(self,name):
        if len(name) >0:
            home = os.environ['HOME'] + "/.bash_profile"
            if not os.path.exists(home):
                return None
            Ropen=open(home,'r')
            for line in Ropen:
                if name + "=" in line:
                    result = re.findall(r'export ' + name + '=\'(.+?)\'',line, re.M|re.I)
                    if len(result) == 1 and '#' not in line:
                        Ropen.close() 
                        return result[0]
                    
            Ropen.close()
            return None
        else:
            faillog("变量名不能为空")
            return None

    def set_system_environment(self,name,value):
        env = self.get_system_environment(name)
        print(env)
        platform = System().get_platform()
        if env != None and env != value:
            warninglog("当前值为：" + env)
            if  platform in ["Windows/Cygwin","Windows"]:
                self.remove_win_env(name,value)
                self.modification_win_env(name,value)
            elif platform in ["Mac OS X","OS/2","OS/2 EMX","Linux"]:
                self.remove_linux_and_macos_env(name,value)
                self.modification_linux_and_macos_env(name,value)
            else:
                faillog("删除环境变量失败，当前平台暂时不支持此功能！")
        elif env != None and  env == value:
            pass
        else:
            if  platform in ["Windows/Cygwin","Windows"]:
                self.modification_win_env(name,value)
            elif platform in ["Mac OS X","OS/2","OS/2 EMX","Linux"]:
                self.modification_linux_and_macos_env(name,value)
            else:
                faillog("修改环境变量失败，当前平台暂时不支持此功能！")

    def remove_system_environment(self,name,value):
        env = self.get_system_environment(name)
        if env == None:
            return
        platform = System().get_platform()
        if  platform in ["Windows/Cygwin","Windows"]:
            self.modification_win_env(name,value)
        elif platform in ["Mac OS X","OS/2","OS/2 EMX","Linux"]:
            self.modification_linux_and_macos_env(name,value)
        else:
            faillog("修改环境变量失败，当前平台暂时不支持此功能！")

    def modification_win_env(self,name,value):
        print("win下修改环境变量")

    def remove_win_env(self,name,value):
        print("win下修改环境变量")

    def modification_linux_and_macos_env(self,name,value):
        try:
            file_data = ""
            home = os.environ['HOME'] + "/.bash_profile"
            if not os.path.exists(home):
                open(home,'w')
            Ropen=open(home,'r')
            for line in Ropen:
                file_data += line
            Ropen.close()
            file_data = file_data + "\nexport " + name + "='" + value + "'\n"

            Wopen=open(home,'w')
            Wopen.write(file_data)
            Wopen.close()

            os.system("source " + home)
            os.environ[name] = value
            successlog("修改环境变量成功")
        except BaseException as error:
            faillog("mac下修改环境变量失败：" + format(error))
            return None
    
    def remove_linux_and_macos_env(self,name,value):
        try:
            file_data = ""
            home = os.environ['HOME'] + "/.bash_profile"
            Ropen=open(home,'r')
            for line in Ropen:
                if name + "=" not in line:
                    file_data += line
            Ropen.close()

            Wopen=open(home,'w')
            Wopen.write(file_data)
            Wopen.close()
            os.unsetenv[name]
            successlog("删除环境变量成功")
            os.system("source " + home)
        except BaseException as error:
            faillog("mac下删除环境变量失败" + format(error))



class System():
    def get_platform(self):
        sys_platform = sys.platform
        if sys_platform == "linux2" :
            return "Linux" 
        elif sys_platform == "win32" :
            return "Windows" 
        elif sys_platform == "cygwin" :
            return "Windows/Cygwin"
        elif sys_platform == "darwin" :
            return "Mac OS X"
        elif sys_platform == "os2" :
            return "OS/2"
        elif sys_platform == "os2emx" :
            return "OS/2 EMX"
        elif sys_platform == "riscos" :
            return "RiscOS"
        elif sys_platform == "atheos" :
            return "AtheOS"

    

if __name__ == "__main__":
    pass
    # SysEvn().set_system_environment('MATCH_PASSWORD','123456')
    # name = "MATCH_PASSWORD"
    # result = re.findall(r'export ' + name + '=\'(.+?)\'',line, re.M|re.I)
    # print(len(result)) 
    # print(SysEvn().get_system_environment("MATCH_PASSWORD"))

    home = '/Users/v-sunweiwei/Desktop/extension/test.rb'
    open(home,'w')
    


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# AUTHOR: XYCoder
# FILE: ~/Desktop//xyscript/xyscript/test/testUI.py
# DATE: 2019/08/21 Wed
# TIME: 17:48:04

# DESCRIPTION:测试模块UI部分

# Tkinter  :是python最简单的图形化模块，总共只有14种组建
# Pyqt     :是python最复杂也是使用最广泛的图形化
# Wx       :是python当中居中的一个图形化，学习结构很清晰
# Pywin    :是python windows 下的模块，摄像头控制(opencv)，常用于外挂制作

import wx
import time,os

from xyscript.test.test import Test
from xyscript.test.test import TestUI
from xyscript.common.sysenv import System
from concurrent.futures import ThreadPoolExecutor, as_completed,ProcessPoolExecutor
from threading import Thread
from threading import Timer
from pubsub import pub
from xyscript.common.xylog import *
import inspect
import ctypes
 


LOG_LEVEL = {'简单':' -v','普通':' -v -v','详细':' -v -v -v'}
DEVICES = {}
REFRESH_DEVICES = {}
CURRENT_SELECT_DEVICE_INDEX = 0
CURRENT_SELECT_PACKAGE_INDEX = 0
CURRENT_INDEX = 1
t = None
t0 = None#主子线程
executor = None #主子线程

FLAT = True #计时线程标志
LAST_FLAT= True #上一次计时线程标志

ui = TestUI()
platform = System().get_platform()

class TestThread(Thread):
    def __init__(self):
  #线程实例化时立即启动
        Thread.__init__(self)
        self.daemon = True

    def run(self):
    #线程执行的代码
        global DEVICES,FLAT,LAST_FLAT
        #print('第二次当前flag：'+ str(FLAT))
        #print('第二次上一次flag：'+ str(LAST_FLAT))
        devices = {}
        for key,value in DEVICES.items():
            packages = []
            if value['is_check'] :
                for package in value['packages']:
                    if package.get('is_check') :
                        packages.append(package['name'])
                devices[key] = packages
        ui.set_config_with_name('mobile',devices)
        Test().setup_task_files(ui)
        run_monkey_with_config(ui)

     
    def _async_raise(self,tid, exctype):
        """raises the exception, performs cleanup if needed"""
        tid = ctypes.c_long(tid)
        if not inspect.isclass(exctype):
            exctype = type(exctype)
        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, ctypes.py_object(exctype))
        if res == 0:
            raise ValueError("invalid thread id")
        elif res != 1:
            # """if it returns a number greater than one, you're in trouble,
            # and you should call it again with exc=NULL to revert the effect"""
            ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, None)
            raise SystemError("PyThreadState_SetAsyncExc failed")
    
    
    def stop_thread(self):
        #print(self.ident)
        self._async_raise(self.ident, SystemExit)
            

def run_monkey_with_config(ui):
    global CURRENT_INDEX,t,executor
    timer_interval = 3
    start_time = time.time()
    executor = ThreadPoolExecutor(max_workers=100)

    mobile_dict = ui.get_config_with_name('mobile')
    array = []
    for key,value in mobile_dict.items():
        array.append(key)
    # 循环次数
    cycles = ui.get_config_with_name('cycles')
    # t = Timer(timer_interval,wx.CallAfter(pub.sendMessage, "update", msg={'current':time.time() - start_time,'cycle':CURRENT_INDEX -1}))#(time.time() - start_time,CURRENT_INDEX -1)
    all_task = [executor.submit(run_adb_str,(ui),(key)) for (index,key) in enumerate(array)]

    if t is None:
        t = Thread(target=update_process, args=(start_time,))
        t.start()
    # else:
    #     t.resume()
    over_num = 0
    for index,future in enumerate(as_completed(all_task)):
        data = future.result()
        if data is None:
            over_num = over_num + 1
            if over_num == len(array):
                if t is not None:
                    t.join()
                CURRENT_INDEX = CURRENT_INDEX + 1
                if CURRENT_INDEX <= cycles:
                    run_monkey_with_config(ui)  

def update_process(start_time):
    global t,CURRENT_INDEX,FLAT,LAST_FLAT
    for i in range(ui.get_config_with_name('life_cycle') * 60 * 60):
        time.sleep(1)

        # #print('当前flag：'+ str(FLAT))
        # #print('上一次flag：'+ str(LAST_FLAT))
        if FLAT and LAST_FLAT:
            wx.CallAfter(pub.sendMessage, "update", msg={'current':i,'cycle':CURRENT_INDEX -1})
        elif FLAT and not LAST_FLAT:

            CURRENT_INDEX = 1
            LAST_FLAT = FLAT
            update_process(start_time)
            break

    

def run_adb_str(ui,key):
    global CURRENT_INDEX
    # 序列号
    dev = key.split('_')[-1]
    # logcat
    logcat_file_name = key + str(CURRENT_INDEX) + '.log'
    logcat_file_name = logcat_file_name.replace(' ','_')
    logcat_file_path = ui.get_config_with_name('logcat_path')
    #print(logcat_file_name)
    #print(logcat_file_path)
    os.popen('adb -s ' + dev + ' shell logcat -c') #清除日志
    os.popen('adb -s ' + dev + ' shell  logcat -v time > ' + logcat_file_path + logcat_file_name) #记录日志
    # 包名
    packages = ui.get_config_with_name('mobile').get(key)
    p_array = ''
    for p in packages:
        p_array = p_array + ' -p ' + p
    # 种子数
    seed_num = ui.get_config_with_name('seed_num')
    # 模式

    implement_way = ui.get_config_with_name('implement_way')
    run_mode = ''
    for behav in implement_way:
        run_mode = run_mode + ' --' + behav
    # 延迟
    time_delay = ui.get_config_with_name('time_delay')
    # 日志等级
    log_level = ui.get_config_with_name('log_level')
    # 执行次数
    run_times = ui.get_config_with_name('run_times')
    # 日志地址
    workspace = ui.get_config_with_name('workspace')
    log_path = workspace + key + '/'  + key + '_' +str(CURRENT_INDEX) + '.log'
    log_path = log_path.replace(' ','-')
    if platform in ["Windows/Cygwin","Windows"]:
        log_path = log_path.replace('/','\\')
    # 关闭手机状态栏
    os.popen('adb -s '+ dev +' shell settings put global policy_control immersive.full=*')
    shell_str = 'adb -s '+ dev +' shell monkey ' + p_array + ' -s ' + str(seed_num) + run_mode + ' --throttle ' + str(time_delay) + ' --pct-touch 40 --pct-motion 25 --pct-appswitch 15 --pct-trackball 5 --pct-majornav 5 ' + log_level + ' ' + str(run_times) + ' >' + log_path + ' 2>&1'
    # #print(shell_str)
    result =os.popen(shell_str)

    # 每次执行周期
    cycle = ui.get_config_with_name('life_cycle')
    run_time_delay = cycle * 60 * 60
    time.sleep(int(run_time_delay))
    Test().kill_monkey_with_package_name(dev)
    Test().kill_logcat(dev) 

class MonkeyFrame(wx.Frame):
    def __init__(self, *args, **kw):
        super().__init__(*args, **kw)

        self.setup_main_window()
        self.panel = wx.Panel(self) 

    def setup_main_window(self):
        # 初始化菜单按钮
        menuBar = wx.MenuBar()
        menu1 = wx.Menu("test")
        menuBar.Append(menu1, "File")
        self.SetMenuBar(menuBar)

        wx.Frame.__init__(self,None,size=(800,600),title='XYMonkey')

        # 工作路径
        wx.StaticText(self,-1,pos=(15,15),label='工作目录:')
        self.workspace_text = wx.TextCtrl(self,-1,pos=(80,10),size=(300,30))
        self.select_workspace = wx.Button(self,-1,label="选择",pos=(400,10),size=(50,30),style = wx.TE_READONLY)

        # 周期(h)
        wx.StaticText(self,-1,pos=(15,60),label='周期(h):')
        self.cycle = wx.SpinCtrl(self, -1, "", (80, 55), (100, 30))  
        self.cycle.SetRange(1,100)  

        # 循环次数
        wx.StaticText(self,-1,pos=(190,60),label='循环次数:')
        self.run_times = wx.SpinCtrl(self, -1, "", (260, 55), (100, 30))  
        self.run_times.SetRange(1,100)  

        # 动作延迟(ms)
        wx.StaticText(self,-1,pos=(370,60),label='动作延迟(ms):')
        self.delay = wx.SpinCtrl(self, -1, "", (470, 55), (100, 30))  
        self.delay.SetRange(1,10000)

        # 总耗时(h)
        wx.StaticText(self,-1,pos=(580,60),label='总耗时(h):')
        self.total_time = wx.SpinCtrl(self, -1, "", (670, 55), (100, 30))  
        self.total_time.SetRange(1,100)  

        # 单次执行次数
        wx.StaticText(self,-1,pos=(15,100),label='单次执行次数:')
        self.times_in_one = wx.StaticText(self,-1,pos=(125,100),label='')

        # 种子数
        wx.StaticText(self,-1,pos=(15,140),label='种子数:')
        self.seed_num = wx.SpinCtrl(self, -1, "", (110, 135), (100, 30))  
        self.seed_num.SetRange(1,10000)

        # 日志等级
        wx.StaticText(self,-1,pos=(15,180),label='日志等级:')
        self.log_level = wx.Choice(self,-1,(110,170),(100, 40))

        # 运行模式

        # 刷新
        self.refresh_btn = wx.Button(self,-1,label='刷新',pos=(20,250),size=(50,30))
        # 手机
        sampleList = []  
        wx.StaticText(self, -1, "设备及应用:", (15, 220),(100,30))  
        self.devices = wx.CheckListBox(self, -1, (110, 220),(300,180),style=1)  

        # 包筛选
        self.package_screen_str = wx.TextCtrl(self,-1,pos=(450,220),size=(230, 25))
        self.screen_btn = wx.Button(self,-1,label="过滤",pos=(700,220),size=(50,30))

        # 应用
        sampleList = []  
        self.packages = wx.CheckListBox(self, -1, (450, 250),(300,150), choices=sampleList)  

        # 日志
        # wx.StaticText(self,-1,pos=(430,95),label='日志:')
        # self.log_str = wx.TextCtrl(self,-1,pos=(480,95),size=(300,300))

        # 进度条
        self.gauge = wx.Gauge(self, -1, 50, pos=(20, 440), size=(760, 25))  
        self.gauge.SetBezelFace(3)  
        self.gauge.SetShadowWidth(3)  
        self.total_process_bar = wx.StaticText(self,-1,label='当前进度: 0%',pos=(15,470),size=(85,30))

        # 取消 确定
        self.cancel_btn = wx.Button(self,-1,label="取消",pos=(660,520),size=(50,30))
        self.confirm_btn = wx.Button(self,-1,label="开始",pos=(730,520),size=(50,30))
        # self.confirm_btn.SetBackgroundColour('#186CD5')

        # 赋值
        self.set_default_for_UI()

        # 事件绑定
        self.Bind(wx.EVT_BUTTON, self.OnButton, self.select_workspace) #文件夹
        self.screen_btn.Bind(wx.EVT_BUTTON,self.screen_btn_click)
        self.cycle.Bind(wx.EVT_SPINCTRL,self.change_cycle)
        self.cycle.Bind(wx.EVT_TEXT,self.change_cycle)
        self.run_times.Bind(wx.EVT_SPINCTRL,self.change_runtimes)
        self.run_times.Bind(wx.EVT_TEXT,self.change_runtimes)

        self.delay.Bind(wx.EVT_SPINCTRL,self.change_delay)
        self.delay.Bind(wx.EVT_TEXT,self.change_delay)

        self.total_time.Bind(wx.EVT_SPINCTRL,self.change_total_time)
        self.total_time.Bind(wx.EVT_TEXT,self.change_total_time)

        self.seed_num.Bind(wx.EVT_SPINCTRL,self.change_seed) #种子数
        self.seed_num.Bind(wx.EVT_TEXT,self.change_seed)

        self.log_level.Bind(wx.EVT_CHOICE,self.One_Play) #日志等级

        self.refresh_btn.Bind(wx.EVT_BUTTON,self.refresh_btn_click)
        self.devices.Bind(wx.EVT_CHECKLISTBOX,self.change_devices_and_packages)
        self.packages.Bind(wx.EVT_CHECKLISTBOX,self.change_devices_and_packages)

        # self.gauge.Bind(wx.EVT_IDLE, self.OnIdle)
        
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Centre()
        self.Show(True)

        self.confirm_btn.Bind(wx.EVT_BUTTON,self.start_monkey)
        self.cancel_btn.Bind(wx.EVT_BUTTON,self.cancel_monkey)

        pub.subscribe(self.change_process, "update")

    def start_and_stop_status(self,stop):
        self.workspace_text.Enable(stop)
        self.select_workspace.Enable(stop)
        self.cycle.Enable(stop)
        self.run_times.Enable(stop)
        self.delay.Enable(stop)
        self.total_time.Enable(stop)
        self.times_in_one.Enable(stop)
        self.seed_num.Enable(stop)
        self.log_level.Enable(stop)
        self.refresh_btn.Enable(stop)
        self.devices.Enable(stop)
        self.screen_btn.Enable(stop)
        self.packages.Enable(stop)
        self.cancel_btn.Enable(not stop)
        self.confirm_btn.Enable(stop)

    def start_monkey(self,event):
        global t,t0,FLAT,LAST_FLAT
        # try:
        self.start_and_stop_status(False)
        LAST_FLAT = FLAT
        FLAT = True
        t0 = TestThread().start()
        # except BaseException as error:
        #     faillog(format(error))
        #     # 停掉所有测试进程
        #     self.cancel_monkey()
        
    def cancel_monkey(self,event):
        global t,executor,t0,FLAT
        self.start_and_stop_status(True)
        mobile = ui.get_config_with_name('mobile')

        # 线程停止
        executor.shutdown(wait=False)
        # 停止计时线程
        if t is not None:
            FLAT = False
            LAST_FLAT = True

        if t0 is not None:
            t0.stop_thread()
        # 停止所有logcat、monkey线程
        # 手机停止
        for key,value in mobile.items():
            dev = key.split('_')[-1]
            Test().kill_monkey_with_package_name(dev)
            Test().kill_logcat(dev)
            # 恢复手机状态栏
            os.popen('adb -s '+ dev +' shell settings put global policy_control null')


    def change_process(self,msg):
        current = msg.get('current')
        cycle = msg.get('cycle')
        #print('current:',current)
        #print('cycle:',cycle)
        cycles = ui.get_config_with_name('cycles')
        cycle_str = '('+ str(cycle) + '/' + str(cycles) +')'
        cycle_time = ui.get_config_with_name('life_cycle')
        cycle_process = round(current / cycle_time / 60 / 60 * 100 /cycles, 2)  +  cycle * 25
        self.total_process_bar.SetLabelText('当前进度: ' + str(cycle_process) + '%')
        self.gauge.SetValue(cycle_process)
 
    def change_devices_and_packages(self,event):
        global DEVICES,CURRENT_SELECT_DEVICE_INDEX,CURRENT_SELECT_PACKAGE_INDEX
        mobile_checks = list(self.devices.GetCheckedItems())
        package_checks = list(self.packages.GetCheckedItems())

        if event.GetEventObject() is self.devices:
            index = self.differ_devices()
            #print('点击了设备index:' + str(index))
            CURRENT_SELECT_DEVICE_INDEX = index
            self.change_devices_check_status()
        elif event.GetEventObject() is self.packages:
            index = self.differ_packages()
            #print('点击了包index:' + str(index))
            CURRENT_SELECT_PACKAGE_INDEX = index
            self.change_devices_check_status()

    def differ_packages(self):
        global DEVICES,CURRENT_SELECT_DEVICE_INDEX,CURRENT_SELECT_PACKAGE_INDEX
        package_checks = list(self.packages.GetCheckedItems())
        index = 0
        packages = []
        for key,value in DEVICES.items():
            if CURRENT_SELECT_DEVICE_INDEX == index:
                packages = value['packages']
                break
            index = index + 1
        # #print(packages)
        package_index = 0
        for package in packages:
            if package.get('is_check') and not self.packages.IsChecked(package_index):
                return package_index
            elif not package.get('is_check') and self.packages.IsChecked(package_index):
                return package_index
            package_index = package_index + 1
        return None

    # {"vivo_vivo X3V_d934619c":{'is_check':false,'packages':[{name:com.xxx.xxx,is_check:true}]}}
    # {"vivo_vivo X3V_d934619c": ["com.mobility.user", "com.mobility.lease.driver"]}
    def change_devices_check_status(self):
        global DEVICES,CURRENT_SELECT_DEVICE_INDEX,CURRENT_SELECT_PACKAGE_INDEX
        index = 0
        mobile = self.devices.IsChecked(CURRENT_SELECT_DEVICE_INDEX)
        package = self.packages.IsChecked(CURRENT_SELECT_PACKAGE_INDEX)
        # #print(mobile)
        # #print(package)
        for key,value in DEVICES.items():
            if index == CURRENT_SELECT_DEVICE_INDEX:
                if mobile is not None:
                    value['is_check'] = mobile
                
                if package is not None:
                    packages = value['packages']
                    package_b = packages[CURRENT_SELECT_PACKAGE_INDEX]
                    package_b['is_check'] = package
                    # #print(package_b)

                if mobile:
                    package_select_array = []
                    package_all_array = []
                    for package_t in value.get('packages'):
                        # #print(package_t)
                        package_all_array.append(package_t.get('name'))
                        if package_t.get('is_check') :
                            package_select_array.append(package_t.get('name'))
                    self.packages.SetItems(package_all_array)
                    self.packages.SetCheckedStrings(package_select_array)

                return

            index = index + 1
        # #print(package)

    def differ_devices(self):
        global DEVICES
        mobile_checks = list(self.devices.GetCheckedItems())
        index = 0
        for key,value in DEVICES.items():
            if value['is_check'] and not self.devices.IsChecked(index):
                return index
            elif not value['is_check'] and self.devices.IsChecked(index):
                return index
            index = index + 1
        return None


    def change_seed(self,e):
        seed_num = self.seed_num.GetValue()
        ui.set_config_with_name('seed_num',seed_num)

    def change_delay(self,event):
        delay = self.delay.GetValue()
        ui.set_config_with_name('time_delay',delay)
        self.change_cycle(None)
        # self.total_time.SetValue(cycle*self.run_times.GetValue())

    def change_total_time(self,event):
        time_use = self.total_time.GetValue()
        cycles = time_use/self.cycle.GetValue()
        self.run_times.SetValue(cycles)
        ui.set_config_with_name('cycles',int(cycles))

    def change_cycle(self,event):
        cycle = self.cycle.GetValue()
        delay = self.delay.GetValue()
        ui.set_config_with_name('life_cycle',cycle)
        self.total_time.SetValue(cycle*self.run_times.GetValue())
        times_in_one = cycle*60*60*1000/delay
        self.times_in_one.SetLabelText(str(int(times_in_one)))
        ui.set_config_with_name('run_times',int(times_in_one))
        # #print(self.cycle.GetValue())

    def change_runtimes(self,event):
        runtimes = self.run_times.GetValue()
        ui.set_config_with_name('cycles',int(runtimes))
        self.total_time.SetValue(runtimes*self.cycle.GetValue())

    def OnPaint(self, e): 
      dc = wx.PaintDC(self)

      brush = wx.Brush("#C0C0C0")
      if platform in ["Windows/Cygwin","Windows"]:
          brush = wx.Brush("#FFFFFF")
      dc.SetBackground(brush) 
      dc.Clear()
		
      font = wx.Font(18, wx.ROMAN, wx.ITALIC, wx.NORMAL) 
      dc.SetFont(font) 
      pen = wx.Pen(wx.Colour(105,105,105)) 
      dc.SetPen(pen) 
      dc.DrawLine(15,47,770,47) 
      dc.DrawLine(15,130,770,130)
      dc.DrawLine(15,130,770,130)

    def set_default_for_UI(self):
        global LOG_LEVEL
        self.workspace_text.SetLabelText(ui.get_config_with_name('workspace'))

        # 种子数
        self.seed_num.SetValue(ui.get_config_with_name('seed_num')) 
        # 周期
        self.cycle.SetValue(ui.get_config_with_name('life_cycle')) 
        # 循环次数
        self.run_times.SetValue(ui.get_config_with_name('cycles')) 
        # 动作延迟
        self.delay.SetValue(ui.get_config_with_name('time_delay'))
        self.change_cycle(None)
        if ui.get_config_with_name('screen_key') is not None and len(ui.get_config_with_name('screen_key')) > 0:
            self.package_screen_str.SetValue(ui.get_config_with_name('screen_key'))
        
        self.total_time.SetValue(self.cycle.GetValue()*self.run_times.GetValue())
        # 日志等级
        self.log_level.AppendItems(list(LOG_LEVEL.keys()))
        # 筛选关键字
        self.refresh_device(True)

    
    def refresh_btn_click(self,e):
        self.refresh_device(True)

    def screen_btn_click(self,event):
        self.refresh_device(False)

    def refresh_device(self,is_first=False):
        global DEVICES,CURRENT_SELECT_DEVICE_INDEX
        if is_first:
            ui.set_config_with_name('screen_key','')
            self.package_screen_str.SetValue('')
        else:
            ui.set_config_with_name('screen_key',self.package_screen_str.GetValue())
       
        screen_word = self.package_screen_str.GetValue()

        mobile = ui.get_all_devices(screen_word)
        keys = list(mobile.keys())
        self.devices.SetItems(keys)
        packages = []
        if len(keys) > 0:
            packages = mobile[keys[CURRENT_SELECT_DEVICE_INDEX]]
            self.packages.SetItems(packages)
        # #print(packages)

        if is_first:
            for key,value in mobile.items():
                device = {}
                device['is_check'] = False
                packages = [] 
                for package in value:
                    package_t = {'name':package,'is_check':False}
                    packages.append(package_t)
                device['packages'] = packages
                DEVICES[key] = device

        else:
            index = 0
            select_array = []
            for key,value in DEVICES.items():
                if value['is_check'] == True:
                    select_array.append(key)
                if index == CURRENT_SELECT_DEVICE_INDEX:
                    package_select_array = []
                    package_device_array = []
                    for package in packages:
                        real_package = {'name':package}
                        for package_1 in value['packages']:
                            if package == package_1['name']:
                                #print(package_1)
                                if package_1['is_check']:
                                    real_package['is_check'] = True
                                    package_select_array.append(package)
                                else:
                                    real_package['is_check'] = False
                                break
                        package_device_array.append(real_package)

                    self.packages.SetCheckedStrings(package_select_array)
                    value['packages'] = package_device_array
                    # #print(DEVICES)
                    break
                
                index = index + 1  
            self.devices.SetCheckedStrings(select_array)

            # 包设置选中状态


    # def OnIdle(self, event):  
    #     self.count = self.count + 1  
    #     if self.count  == 50:  
    #         self.count = 0 
    #     self.gauge.SetValue(self.count)  

    def One_Play(self,event):
        log_level = LOG_LEVEL[event.GetString()]
        ui.set_config_with_name('log_level',log_level)


    def OnButton(self, event):
        dlg = wx.DirDialog(self,u"选择文件夹",ui.get_config_with_name('workspace'),style=wx.DD_DEFAULT_STYLE)
        if dlg.ShowModal() == wx.ID_OK:
            self.workspace_text.SetLabelText(dlg.GetPath() + '/')
            ui.set_config_with_name('workspace',dlg.GetPath() + '/')
            if platform in ["Windows/Cygwin","Windows"]:
                self.workspace_text.SetLabelText(dlg.GetPath() + '\\')
                ui.set_config_with_name('workspace',dlg.GetPath() + '\\')
        dlg.Destroy()


class TestUI:
    def start_test(self):
        frame = wx.App()
        app = MonkeyFrame()
        app.Show()
        frame.MainLoop()

def main():
    TestUI().start_test()


if __name__ == '__main__':
    main()


import sys,os
 # print("\033[1;31m failed \033[0m")
    # print("\033[1;32m success \033[0m")
def faillog(content):
    print("\033[1;31m" + str(content) + "\033[0m")

def successlog(content):
    print("\033[1;32m" + str(content) + "\033[0m")

def warninglog(content):
    print("\033[1;33m" + str(content) + "\033[0m")

def printandresult(command, print_msg=True):
    import subprocess
    p = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    lines = []
    for line in iter(p.stdout.readline, b''):
        line = line.rstrip().decode('utf8')
        if print_msg:
            print(line)
        lines.append(line)
    return ','.join(lines)

def printandresult_shell(shell):
    import subprocess
    cmd = subprocess.Popen(shell, stdin=subprocess.PIPE, stderr=sys.stderr, close_fds=True,
                           stdout=sys.stdout, universal_newlines=True, shell=True, bufsize=1)

    cmd.communicate()
    return cmd.returncode
class logitem:
    def failitem(self,content,bold=False):
        boldstr = "1" if bold else "0"
        return "\033["+ boldstr +";31m" + str(content) + "\033[0m"

    def successitem(self,content,bold=False):
        boldstr = "1" if bold else "0"
        return "\033["+ boldstr +";32m" + str(content) + "\033[0m"
    
    def warningitem(self,content,bold=False):
        boldstr = "1" if bold else "0"
        return "\033["+ boldstr +";33m" + str(content) + "\033[0m"

class Log:
    def __init__(self, filename="log.text"):
        self.terminal = sys.stdout
        self.log = open(filename, "a")
 
    def write(self, message):
        self.terminal.write(message)
        self.log.write(message)
 
    def flush(self):
        pass

    def save_log_in_lines(self,lines=None):
        pass


diagnosemail_header = \
    """
    <head>
        <title></title>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1">
        <style>
            .content{
                padding: 20px;
                border:10px solid rgb(108, 172, 218)
            }
            .file-content{
                display: inline-block;
            }
            .file-content span{
                display: block;
                text-align:center;
            }
            .file-content a{
                text-decoration: none;
                color:black;
            }
            .result_table{
                width:100%;
                border:0;
                cellspacing:1; 
                cellpadding:0;
                border-collapse: collapse;
            }
            .table-header td{
                color: white;
            }
            table td{
                padding-left: 10px;
            }
            table,table tr th, table tr td {
                border:1px solid #ccc;
                }
            table tr td a{
                text-decoration: none;
                color:#1b69b6;
            }
            #result-text{
                width: 70%;
                resize: none;
            }
        </style>
    </head>
    """

diagnosemail_body = \
    """
    <body>
        <div class="content">
            <h4>您好：</h4>
            <h4>&nbsp;&nbsp;&nbsp;&nbsp;针对您最近一次的提交，诊断结果如下：</h4>
            <table class="result_table" >
                <tr class="table-header" style="background-color: gray">
                    <td>名称</td>
                    <td>内容</td>
                    <td>备注</td>
                </tr>
                <tr>
                    <td>project name</td>
                    <td><a href="{project_url}">{project_name}</a></td>
                    <td>&nbsp;</td>
                </tr>
                <tr>
                    <td>commit user</td>
                    <td>{user_name}</td>
                    <td>&nbsp;</td>
                </tr>
                <tr>
                    <td>branch</td>
                    <td>{branch_name}</td>
                    <td>&nbsp;</td>
                </tr>
                <tr>
                    <td>commit</td>
                    <td>{commit_content}</td>
                    <td>&nbsp;</td>
                </tr>
                <tr>
                    <td>commit id</td>
                    <td><a href="{commit_url}">{commit_id}</a></td>
                    <td>&nbsp;</td>
                </tr>
                <tr>
                    <td>commit date</td>
                    <td>{date}</td>
                    <td>&nbsp;</td>
                </tr>
                <tr>
                    <td>diagnose result</td>
                    <td>{result}</td>
                    <td>&nbsp;</td>
                </tr>
            </table>
            <h4 >diagnose detail:</h4>
            <textarea name="" id="result-text" rows="10" readonly>{result_detail}</textarea>
            <h4>更多详情见附件</h4>
            <img src="cid:image1" alt="" style="display:none;">
        </div>
    </body>
    """

mergemail_body = \
        """
        <body>
        <div class="content">
            <h4>您好：</h4>
            <h4>&nbsp;&nbsp;&nbsp;&nbsp;针对iOS项目最近一次的代码合并，结果如下：</h4>
            <h5>项目名称：{project_name}</h5>
            <h5>合并分支：{package_branch}</h5>
            <h5>本次合并时间：{start_time}</h5>
            <h5>上一次合并时间：{end_time}</h5>

            <H3>壳工程</H3>
            <table class="result_table_shell" >
                <tr class="table-header" style="background-color: gray">
                    <td>提交人</td>
                    <td>提交时间</td>
                    <td>提交内容</td>
                    <td>备注</td>
                </tr>
                <&shell_list&>
            </table>
            <&submodule_list&>
            <&private_modules&>
        </div>
    </body>
        """

if __name__ == "__main__":
    print(diagnosemail_body)



#!/usr/bin/env python
# coding=utf-8
#-*- encoding:utf-8 -*-
# from __future__ import print_function
import configparser, os, sys, re
from xyscript.common.xylog import logitem,warninglog
from xyscript.common.interaction import Interaction
from xyscript.crawler.version import Version

#缓存文件存储路径
log_store_local = "/dist/" 

# root_dir = os.path.abspath('.')
dir = os.path.split(os.path.realpath(__file__))[0]
root_dir = os.path.abspath(os.path.join(dir,"."))
pro_dir = os.path.abspath(os.path.join(dir,".."))
cf = configparser.ConfigParser()
cf.read(root_dir + '/config.ini')
secs = cf.sections()

class Config:
    def get_version(self):
        global cf
        version = cf.get("Project-config", "version")
        return version
    
    def get_description(self):
        global cf
        description = cf.get("Project-config", "description")
        return description

    def get_long_description(self):
        global pro_dir
        long_des = open(os.path.join(pro_dir,'README.rst')).read()
        return long_des

    def get_file_store_path(self):
        global cf
        path = cf.get("File-store-path", "log_store_local")
        return path

    def get_sys_root_path(self):
        return os.path.expanduser('~')

    def get_config(self,kind,key):
        global cf
        path = cf.get(kind,key)
        return path
    
    def check_version(self):
        """ 检查版本号 """
        # "RuntimeError: PyPI's XMLRPC API has been temporarily disabled due to unmanageable load and will be deprecated in the near future. See https://status.python.org/ for more information.
        # shell_str = "pip3 search xyscript"
        # shell_str_brew = "brew info xyscript"
        # result = os.popen(shell_str)
        # text = result.read()
        # result = re.findall(r"xyscript \((.+?)\)",text)

        # if len(result) == 1:
        local_version = self.get_version()
        remote_version = Version.get_last_version()
        # print(remote_version)
        # lines,columns = os.popen('stty size','r').readlines()[0].replace('\n','').split(' ')
        # cols = '#'*int(columns)
        print(local_version)
        print(remote_version)
        if local_version != remote_version:
            warninglog("###########################################################################################")
            warninglog("# the latest version of xyscript is:" + remote_version + ", but yours is still:" + local_version)
            warninglog("# this will cause errors or some features will not work")
            warninglog("# you can use 'pip3 install xyscript -U' to update the latest version")
            warninglog("###########################################################################################")
        

if __name__ == "__main__":
    # Config().check_version()
    # print(Interaction().get_user_input("please enter password"))
    # print(Config().get_sys_root_path())
    Config().check_version()
    # lines,columns = os.popen('stty size','r').readlines()[0].replace('\n','').split(' ')
    # print(columns)




#!/usr/bin/env python
#-*- encoding:utf-8 -*-
# coding=utf-8

from setuptools import setup, find_packages
from xyscript.config import Config

__author__ = 'XYCoder'
__date__ = '2019/04/18'

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
    extra = {'scripts': ["bin/xyscript"]}
else:
    extra = {
        'test_suite': 'xyscript.test',
        'entry_points': {
            'console_scripts': ['xyscript = xyscript.api:main'],
        },
    }

data_files = [('/xyscript',['config.ini'])]

setup(
    name='xyscript',
    version=Config().get_version(),
    description=Config().get_description(),
    long_description=Config().get_long_description(),
    author='XYCoder',
    author_email='m18221031340@163.com',
    maintainer='XYCoder',
    maintainer_email='m18221031340@163.com',
    license='BSD License',
    packages=find_packages(),
    # data_files=data_files,
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
    platforms=["all"],
    url='https://www.baidu.com',
    install_requires=['requests', 'GitPython', 'setuptools','configparser','progressbar','wxPython','Pypubsub','bs4'],
    include_package_data=True,
    classifiers=[
        "Development Status :: 6 - Mature",
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        'Topic :: Software Development :: Libraries'
    ],
**extra)

if __name__ == "__main__":
    pass
    # Config().get_description()
    # print(setup)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from xyscript.config import Config
import stat
from xyscript.common.xylog import *
from xyscript.common.interaction import Interaction
import zipfile
import itertools as its
from progressbar import *
from xyscript.common.timeusage import func_time
from threading import Thread
from concurrent.futures import ThreadPoolExecutor, as_completed

class File:
    def compress_file_with_password(self,path,password):
        file_name = path.split("/")[-1]
        zip_file_name = ".".join(file_name.split(".")[0:-1]) + ".zip"
        shell = "zip -re " + zip_file_name 
        os.system(shell)
    
    @func_time
    def uncompress_file(self,path):
        try:
            file_name = path.split("/")[-1]
            sup_path = path[0:-(len(file_name))]
            pwd_file_name = ".".join(file_name.split(".")[0:-1]) + ".txt"
            pwd_file_path = sup_path + pwd_file_name
            zf = self.get_zfile(file_name,path)
            self.make_pw_file(pwd_file_path,zf)
            

            # pwd_file = open(pwd_file_path)

            # executor = ThreadPoolExecutor(max_workers=10)
            # all_task = [executor.submit(self.extractFile,(zf),(line.strip('\n')),(index)) for (index,line) in enumerate(pwd_file.readlines()[0:-2])]
            # for future in as_completed(all_task):
            #     data = future.result()
            #     if data is not None:
            #         #print(format(data))
            # pwd_file.close()
        except BaseException as error:
            faillog("uncompress file failed:" + format(error))

    def get_zfile(self,file_name,path):
        zf = None
        if file_name.endswith(".zip"):
            zf = zipfile.ZipFile(path)
        elif file_name.endswith(".rar"):
            try:
                from unrar import rarfile
            except:
                path = 'pip3 install --upgrade pip'
                os.system(path)
                path = 'pip3 install unrar'
                os.system(path)
                from unrar import rarfile
                zf = rarfile.RarFile(path)
        return zf
        
    def extractFile(self,zfile,password,current):
        try:
            zfile.extractall(pwd=password.encode('utf-8'))
            return 'Found password :'+ password +'\n'
        except:
            return None

    def make_pw_file(self,path,zfile):
        num_array = "1234567890"
        capital_case_array = "ZXCVBNMASDFGHJKLQWERTYUIOP"
        lower_case_array = "zxcvbnmasdfghjklqwertyuiop"
        special_character_array = "~!@#$%^&*()_+;:\'\",./<>?\|\b"

        inter = Interaction()
        num = inter.get_user_input("是否存在数字(y/n)")
        capital_case = inter.get_user_input("是否存在大写字母(y/n)")
        lower_case = inter.get_user_input("是否存在小写字母(y/n)")
        special_character = inter.get_user_input("是否存在特殊字符(y/n)")
        max_length = inter.get_user_input("最大长度")
        min_length = inter.get_user_input("最小长度")
        repeated_cha = inter.get_user_input("重复字符个数")
        
        words = ""
        if num == "y":
            words = words + num_array
        if capital_case == "y":
            words = words + capital_case_array
        if lower_case == "y":
            words = words + lower_case_array
        if special_character == "y":
            words = words + special_character_array

        words = "10245678WEIwei"
        
        maxlen = 12
        minlen = 1
        if max_length is not None:
            maxlen = int(max_length)
        if min_length is not None:
            minlen = int(min_length)

        try:
            # dic = open(path,"a")
            # #清空文件
            # dic.seek(0)
            # dic.truncate()
            # warninglog("This will take a few minutes. Wait a moment")
            executor = ThreadPoolExecutor(max_workers=100)
            for count in range(minlen,maxlen +1):
                #print("\n正在生成 %d 位密码，时间可能有点长，请耐心等待..." %(count))
                r = its.product(words,repeat=count)

                array = []
                for i in r:
                    array.append("".join(i))

                all_task = [executor.submit(self.extractFile,(zfile),(line),(index)) for (index,line) in enumerate(array)]

                for index,future in enumerate(as_completed(all_task)):
                    sys.stdout.write("\r正在尝试: %d 位数，进度:%d / %d" %(count,index+1, len(all_task)))
                    sys.stdout.flush()
                    data = future.result()
                    if data is not None:
                        #print("\n %s" %(format(data)))
                        return
                    # dic.write("".join(i))
                    # dic.write("\n")
            # dic.close()
            #print("\n")
            warninglog("Please confirm the correct rules")
        except BaseException as error:
            #print("\n")
            faillog("make password file failed:" + format(error))


    def progressbar(self,nowprogress,total):
        get_progress = int((nowprogress + 1) * (50/total))
        get_pro = int(50 - get_progress)
        percent = (nowprogress + 1) * (100/total)
        #print(" " + "[" + ">" + get_progress + "-" + get_pro + ']' + "%.2f" % percent + "%")

    def find_files_contain(self,path,name):
        """找到目录下包含name的所有文件"""
        result_array = []
        for root, dirs, files in os.walk(path):
            for file in files:
                if name in file:
                    result_array.append(os.path.join(root, file))
        return result_array

    def creat_folder(self,path):
        from pathlib import Path

        try:
            #创建目录
            if Path(path).exists():
                # warninglog(path + " is exists")
                pass
            else:
                os.makedirs(path)
                # successlog("creat folder success:" + path)

            # 更改权限
            os.chmod(path, stat.S_IRWXU)
        except BaseException as error:
            faillog(format(error))


if __name__ == "__main__":
    File().uncompress_file("/Users/v-sunweiwei/Desktop/extension/test/1.rar")

    # File().creat_folder("~/xycache/test")
    # File().make_pw_file("/Users/v-sunweiwei/Desktop/extension/test/ziptest.txt")

    # path = '"'+os.path.dirname(sys.executable)+'\\scripts\\pip3" install --upgrade pip'
    # #print(path)
    # for i in range(1000):
    #     percent = 1.0 * i / 1000 * 100
    #     sys.stdout.write("\r 你好: %d / %d" %(percent, 100))
    #     sys.stdout.flush()
    #     time.sleep(0.01)
    # #print("\n")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os,sys
from xyscript.common.xylog import *
from xyscript.config import Condig

def upload_to_pypi():
    print("upload_to_pypi")
    try:
        
        package_wheel_shell = 'python setup.py sdist bdist_wheel'
        version = Config().get_version()
        upload_shell = 'twine upload dist/xyscript-' + version + '.tar.gz'
        printandresult(package_wheel_shell)
        text = printandresult(upload_shell)
        successlog('package to pypi success!')
    except BaseException as error:
        faillog('package to pypi failed!')

def main():
    pass

if __name__ == '__main__':
    main()
    
    